/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

/*******************************************************************************
 * DWT: SWT port for the D Language
 * 
 * Contributors:
 * 	The DWT team at http://www.dsource.org/projects/dwt
 * 
 *******************************************************************************/



module dwt.graphics.point;


// import org.eclipse.swt.internal.SerializableCompatibility;

/**
 * Instances of this class represent places on the (x, y)
 * coordinate plane.
 * <p>
 * The coordinate space for rectangles and points is considered
 * to have increasing values downward and to the right from its
 * origin making this the normal, computer graphics oriented notion
 * of (x, y) coordinates rather than the strict mathematical one.
 * </p>
 * <p>
 * Application code does <em>not</em> need to explicitly release the
 * resources managed by each instance when those instances are no longer
 * required, and thus no <code>dispose()</code> method is provided.
 * </p>
 *
 * @see Rectangle
 */

public final class Point /* implements SerializableCompatibility */ {
	
private import dwt.util.util;
private import std.string;

	/**
	 * the x coordinate of the point
	 */
	public int x;
	
	/**
	 * the y coordinate of the point
	 */
	public int y;
	
/**
 * Constructs a new point with the given x and y coordinates.
 *
 * @param x the x coordinate of the new point
 * @param y the y coordinate of the new point
 */
public this(int x, int y) {
	this.x = x;
	this.y = y;
}

/**
 * Compares the argument to the receiver, and returns true
 * if they represent the <em>same</em> object using a class
 * specific comparison.
 *
 * @param object the object to compare with this object
 * @return <code>true</code> if the object is the same as this object and <code>false</code> otherwise
 *
 * @see #hashCode
 */
public bit equals (Object object) {
	if (object == this) return true;
	// JJR: I could also have used thd D classinfo method, but this
	// seems to accomplish much the same thing but with less work.
	// alternative is similar to java version but compares class names
	// with object.classinfo.name == this.classinfo.name.  We'll see
	// if this is acceptable.
	// BSA: the original 'Point p = (Point)object;' didn't work, nor did
	// adding cast(Point) - so I commented out for now.
	Point p = cast(Point)object;  /+ TODO: fix this (see comments above) +/
	if (!p) return false;   // indicates p is not a Point 
	return (p.x == this.x) && (p.y == this.y);
}

/**
 * Returns an integer hash code for the receiver. Any two 
 * objects which return <code>true</code> when passed to 
 * <code>equals</code> must return the same value for this
 * method.
 *
 * @return the receiver's hash
 *
 * @see #equals
 */
public int hashCode () {
	return x ^ y;
}

/**
 * Returns a string containing a concise, human-readable
 * description of the receiver.
 *
 * @return a string representation of the point
 */

/+  TODO: compiler complains: 'dwt\graphics\point.d(97): function toString overrides but is not covariant with toString'  +/
public char[] toString () {
	char[] pointDescription = "Point {"; // TODO: <Blandger> DMD complains, why??
	//~ .toString(x) ~ ", " ~ .toString(y) ~ "}";
	return pointDescription;
}

}

