/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

/*******************************************************************************
 * DWT: SWT port for the D Language
 *
 * Contributors:
 * 	The DWT team at http://www.dsource.org/projects/dwt
 *
 *******************************************************************************/



module dwt.internal.win32.os;

private
{
    import dwt.internal.library;

    import dwt.internal.win32.tchar;
    import dwt.internal.win32.types;
}

public class OS {

	/*
	 * DWT Windows flags.
	 */
	static int IsWin32s;
	const int IsWin95 = 0;
	const int IsWinNT = 1;
	const int IsWinCE = 0;
	static int IsPPC;
	static int IsHPC;
	static int IsSP;
	static int IsDBLocale;
	const int IsUnicode = 1;
	static int WIN32_MAJOR, WIN32_MINOR;

	/*
	* Flags for Window API GetVersionEx
	*/
	public const int VER_PLATFORM_WIN32s = 0;
	public const int VER_PLATFORM_WIN32_WINDOWS = 1;
	public const int VER_PLATFORM_WIN32_NT = 2;
	public const int VER_PLATFORM_WIN32_CE = 3;

	/*
	* Initialize the Windows flags
	*/
version (NO)
{
    static this () {

	    /* Load the DWT library */
	    Library.loadLibrary("dwt"); //$NON-NLS-1$

	    /*
	    * Try the UNICODE version of GetVersionEx first
	    * and then the ANSI version.  The UNICODE version
	    * is present on all versions of Windows but is not
	    * implemented on Win95/98/ME.
	    *
	    * NOTE: The value of OSVERSIONINFO.sizeof cannot
	    * be static final because it relies on the Windows
	    * platform version to be initialized and IsUnicode
	    * has not been calculated.  It must be initialized
	    * here, after the platform is determined in order
	    * for the value to be correct.
	    */
	    OSVERSIONINFO* info;


	    // TEMPORARY CODE
	    char[]MBCS = System.getProperty ("MBCS"); //$NON-NLS-1$
	    if (MBCS != null) {
		    //info = new OSVERSIONINFOA ();
		    info.dwOSVersionInfoSize = OSVERSIONINFOA.sizeof;
		    OS.GetVersionExA (cast(OSVERSIONINFOA*)&info);
	    } else {
		    //info = new OSVERSIONINFOW ();
		    info.dwOSVersionInfoSize = OSVERSIONINFOW.sizeof;
		    if (!OS.GetVersionExW (cast(OSVERSIONINFOW*)&info)) {
			    info = new OSVERSIONINFOA ();
			    info.dwOSVersionInfoSize = OSVERSIONINFOA.sizeof;
			    OS.GetVersionExA (cast(OSVERSIONINFOA*)&info);
		    }
	    }
	    //OSVERSIONINFO.sizeof = info.dwOSVersionInfoSize;

	/+ some C code found on the web that may help
	    //http://www.flipcode.com/tutorials/tut_strings01.shtml

	    OSVERSIONINFOW		info;
	    //memset(&info, 0, sizeof(OSVERSIONINFOW));
	    os.dwOSVersionInfoSize = OSVERSIONINFOW.size;
	    return (GetVersionExW(&info) != 0);
	+/
    }

    /+ TODO: assuming Unicode, and use W functions +/

    this() {
	    IsUnicode = 1;

	    IsWin32s = 0; //info.dwPlatformId == VER_PLATFORM_WIN32s;
	    IsWin95 = 0; //info.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS;
	    IsWinNT = 1; //info.dwPlatformId == VER_PLATFORM_WIN32_NT;
	    IsWinCE = 0; //info.dwPlatformId == VER_PLATFORM_WIN32_CE;
    	IsSP = IsSP();
    	IsPPC = IsPPC();
    	IsHPC = IsWinCE && !IsPPC && !IsSP;
    //	WIN32_MAJOR = info.dwMajorVersion;
    //	WIN32_MINOR = info.dwMinorVersion;

    /+
	    // TEMPORARY CODE
	    if (MBCS != null) {
		    IsUnicode = false;
		    System.out.println ("*** DWT - Warning: Unicode disabled"); //$NON-NLS-1$
	    } else {
		    IsUnicode = !IsWin32s && !IsWin95;
	    }

	    /* Get the DBCS flag */
	    int index = 0;
	    while (index <= 0xFF) {
		    if (OS.IsDBCSLeadByte ((byte) index)) break;
		    index++;
	    }
	    IsDBLocale = index <= 0xFF;

    +/

    } // end this() ctor
}

//	}  // from the static block of Java



	/* Flag used on WinCE */
	const int SYS_COLOR_INDEX_FLAG = IsWinCE ? 0x40000000 : 0x0;

	/*
	* NOTE:  There is a bug in JVM 1.2 where loading
	* a class with a large number of constants causes
	* a segment fault to occur sometime later after
	* the class is loaded.  The fix is to break the
	* class up into a hierarchy of classes that each
	* contain a smaller number of constants.  This
	* fix is not necessary at this time but is required
	* when all constants are uncommented.  We have not
	* done the research to determine the limit.
	*/

	/+ DWT notes
	 +
	 +	First, compiler complained about some of the 0xff000000 values and implicitly
	 +	casting from uint to int.  So, we changed the const declare to a uint.
	 +
	 +	As of dmd 0.89, we want to just do a const int XXX = cast(int)0xff000000;
	 +	2004.5.23 BSA
	 +
	 +/
	/* Constants */
	public const int ALTERNATE = 1;
	public const int BFFM_INITIALIZED = 0x1;
	public const int BFFM_SETSELECTION = IsUnicode ? 0x467 : 0x466;
	public const int BFFM_VALIDATEFAILED = IsUnicode ? 0x4 : 0x3;
	public const int BFFM_VALIDATEFAILEDW = 0x4;
	public const int BFFM_VALIDATEFAILEDA = 0x3;
	public const int BF_BOTTOM = 0x8;
	public const int BF_RIGHT = 0x4;
	public const int BIF_EDITBOX = 0x10;
	public const int BIF_NEWDIALOGSTYLE = 0x40;
	public const int BIF_RETURNONLYFSDIRS = 0x1;
	public const int BIF_VALIDATE = 0x20;
	public const int BITSPIXEL = 0xc;
	public const int BI_BITFIELDS = 3;
	public const int BI_RGB = 0;
	public const int BLACKNESS = 0x42;
	public const int BM_CLICK = 0xf5;
	public const int BM_GETCHECK = 0xf0;
	public const int BM_SETCHECK = 0xf1;
	public const int BM_SETIMAGE = 0xf7;
	public const int BM_SETSTYLE = 0xf4;
	public const int BN_CLICKED = 0x0;
	public const int BN_DOUBLECLICKED = 0x5;
	public const int BST_CHECKED = 0x1;
	public const int BST_UNCHECKED = 0x0;
	public const int BS_BITMAP = 0x80;
	public const int BS_CENTER = 0x300;
	public const int BS_CHECKBOX = 0x2;
	public const int BS_DEFPUSHBUTTON = 0x1;
	public const int BS_FLAT = 0x8000;
	public const int BS_GROUPBOX = 0x7;
	public const int BS_ICON = 0x40;
	public const int BS_LEFT = 0x100;
	public const int BS_NOTIFY = 0x4000;
	public const int BS_OWNERDRAW = 0xb;
	public const int BS_PUSHBUTTON = 0x0;
	public const int BS_PUSHLIKE = 0x1000;
	public const int BS_RADIOBUTTON = 0x4;
	public const int BS_RIGHT = 0x200;
	public const int BS_SOLID = 0x0;
	public const int BTNS_AUTOSIZE = 0x10;
	public const int BTNS_BUTTON = 0x0;
	public const int BTNS_CHECK = 0x2;
	public const int BTNS_CHECKGROUP = 0x6;
	public const int BTNS_DROPDOWN = 0x8;
	public const int BTNS_GROUP = 0x4;
	public const int BTNS_SEP = 0x1;
	public const int CBN_EDITCHANGE = 0x5;
	public const int CBN_KILLFOCUS = 0x4;
	public const int CBN_SELCHANGE = 0x1;
	public const int CBN_SETFOCUS = 0x3;
	public const int CBS_AUTOHSCROLL = 0x40;
	public const int CBS_DROPDOWN = 0x2;
	public const int CBS_DROPDOWNLIST = 0x3;
	public const int CBS_NOINTEGRALHEIGHT = 0x400;
	public const int CBS_SIMPLE = 0x1;
	public const int CB_ADDSTRING = 0x143;
	public const int CB_DELETESTRING = 0x144;
	public const int CB_ERR = cast(int)0xffffffff;
	public const int CB_ERRSPACE = cast(int)0xfffffffe;
	public const int CB_FINDSTRINGEXACT = 0x158;
	public const int CB_GETCOUNT = 0x146;
	public const int CB_GETCURSEL = 0x147;
	public const int CB_GETDROPPEDCONTROLRECT = 0x152;
	public const int CB_GETDROPPEDSTATE = 0x157;
	public const int CB_GETEDITSEL = 0x140;
	public const int CB_GETITEMHEIGHT = 0x154;
	public const int CB_GETLBTEXT = 0x148;
	public const int CB_GETLBTEXTLEN = 0x149;
	public const int CB_INSERTSTRING = 0x14a;
	public const int CB_LIMITTEXT = 0x141;
	public const int CB_RESETCONTENT = 0x14b;
	public const int CB_SELECTSTRING = 0x14d;
	public const int CB_SETCURSEL = 0x14e;
	public const int CB_SETEDITSEL = 0x142;
	public const int CB_SHOWDROPDOWN = 0x14f;
	public const int CCM_FIRST = 0x2000;
	public const int CCM_SETBKCOLOR = 0x2001;
	public const int CCS_NODIVIDER = 0x40;
	public const int CCS_NORESIZE = 0x4;
	public const int CC_ANYCOLOR = 0x100;
	public const int CC_ENABLEHOOK = 0x10;
	public const int CC_RGBINIT = 0x1;
	public const int CDDS_POSTERASE = 0x00000004;
	public const int CDDS_PREERASE = 0x00000003;
	public const int CDDS_PREPAINT = 0x00000001;
	public const int CDDS_ITEM = 0x00010000;
	public const int CDDS_ITEMPREPAINT = CDDS_ITEM | CDDS_PREPAINT;
	public const int CDDS_SUBITEM = 0x00020000;
	public const int CDRF_DODEFAULT = 0x00000000;
	public const int CDRF_NEWFONT = 0x00000002;
	public const int CDRF_NOTIFYITEMDRAW = 0x00000020;
	public const int CDRF_NOTIFYPOSTERASE = 0x00000040;
	public const int CDRF_NOTIFYSUBITEMDRAW = 0x00000020;
	public const int CFE_AUTOCOLOR = 0x40000000;
	public const int CFE_ITALIC = 0x2;
	public const int CFE_STRIKEOUT = 0x8;
	public const int CFE_UNDERLINE = 0x4;
	public const int CFM_BOLD = 0x1;
	public const int CFM_CHARSET = 0x8000000;
	public const int CFM_COLOR = 0x40000000;
	public const int CFM_FACE = 0x20000000;
	public const int CFM_ITALIC = 0x2;
	public const int CFM_SIZE = cast(int)0x80000000;
	public const int CFM_STRIKEOUT = 0x8;
	public const int CFM_UNDERLINE = 0x4;
	public const int CFM_WEIGHT = 0x400000;
	public const int CFS_POINT = 0x2;
	public const int CF_EFFECTS = 0x100;
	public const int CF_INITTOLOGFONTSTRUCT = 0x40;
	public const int CF_SCREENFONTS = 0x1;
	public const int CF_TEXT = 0x1;
	public const int CF_UNICODETEXT = 13;
	public const int CF_USESTYLE = 0x80;
	public const int CLR_DEFAULT = cast(int)0xff000000;
	public const int CLR_INVALID = cast(int)0xffffffff;
	public const int COLORONCOLOR = 0x3;
	public const int COLOR_3DDKSHADOW = 0x15 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_3DFACE = 0xf | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_3DHIGHLIGHT = 0x14 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_3DHILIGHT = 0x14 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_3DLIGHT = 0x16 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_3DSHADOW = 0x10 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_ACTIVECAPTION = 0x2 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_BTNFACE = 0xf | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_BTNHIGHLIGHT = 0x14 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_BTNSHADOW = 0x10 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_BTNTEXT = 0x12 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_CAPTIONTEXT = 0x9 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_GRADIENTACTIVECAPTION = 0x1b | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_GRADIENTINACTIVECAPTION = 0x1c | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_HIGHLIGHT = 0xd | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_HIGHLIGHTTEXT = 0xe | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_INACTIVECAPTION = 0x3 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_INACTIVECAPTIONTEXT = 0x13 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_INFOBK = 0x18 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_INFOTEXT = 0x17 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_MENU = 0x4 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_MENUTEXT = 0x7 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_SCROLLBAR = 0x0 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_WINDOW = 0x5 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_WINDOWFRAME = 0x6 | SYS_COLOR_INDEX_FLAG;
	public const int COLOR_WINDOWTEXT = 0x8 | SYS_COLOR_INDEX_FLAG;
	public const int COMPLEXREGION = 0x3;
	public const int CP_ACP = 0x0;
	public const int CP_INSTALLED = 0x1;
	public const int CS_BYTEALIGNWINDOW = 0x2000;
	public const int CS_DBLCLKS = 0x8;
	public const int CS_GLOBALCLASS = 0x4000;
	public const int CS_HREDRAW = 0x2;
	public const int CS_VREDRAW = 0x1;
	public const int CW_USEDEFAULT = cast(int)0x80000000;
	public const int DCX_CACHE = 0x2;
	public const int DCX_CLIPCHILDREN = 0x8;
	public const int DCX_CLIPSIBLINGS = 0x10;
	public const int DEFAULT_CHARSET = 0x1;
	public const int DEFAULT_GUI_FONT = 0x11;
	public const int DFCS_BUTTONCHECK = 0x0;
	public const int DFCS_CHECKED = 0x400;
	public const int DFCS_FLAT = 0x4000;
	public const int DFCS_INACTIVE = 0x100;
	public const int DFCS_PUSHED = 0x200;
	public const int DFCS_SCROLLDOWN = 0x1;
	public const int DFCS_SCROLLLEFT = 0x2;
	public const int DFCS_SCROLLRIGHT = 0x3;
	public const int DFCS_SCROLLUP = 0x0;
	public const int DFC_BUTTON = 0x4;
	public const int DFC_SCROLL = 0x3;
	public const int DIB_RGB_COLORS = 0x0;
	public const int DISP_E_EXCEPTION = cast(int)0x80020009;
	public const int DI_NORMAL = 0x3;
	public const int DLGC_BUTTON = 0x2000;
	public const int DLGC_HASSETSEL = 0x8;
	public const int DLGC_STATIC = 0x100;
	public const int DLGC_WANTALLKEYS = 0x4;
	public const int DLGC_WANTARROWS = 0x1;
	public const int DLGC_WANTCHARS = 0x80;
	public const int DLGC_WANTTAB = 0x2;
	public const int DM_SETDEFID = 0x401;
	public const int DSS_DISABLED = 0x20;
	public const int DSTINVERT = 0x550009;
	public const int DST_BITMAP = 0x4;
	public const int DST_ICON = 0x3;
	public const int DT_CALCRECT = 0x400;
	public const int DT_EDITCONTROL = 0x2000;
	public const int DT_EXPANDTABS = 0x40;
	public const int DT_LEFT = 0x0;
	public const int DT_NOPREFIX = 0x800;
	public const int DT_SINGLELINE = 0x20;
	public const int DT_WORDBREAK = 0x10;
	public const int ECOOP_AND = 0x3;
	public const int ECOOP_OR = 0x2;
	public const int ECO_AUTOHSCROLL = 0x80;
	public const int EDGE_ETCHED = 0x6;
	public const int EDGE_SUNKEN = 0xa;
	public const int EM_CANUNDO = 0xc6;
	public const int EM_CHARFROMPOS = 0xd7;
	public const int EM_DISPLAYBAND = 0x433;
	public const int EM_GETFIRSTVISIBLELINE = 0xce;
	public const int EM_GETLIMITTEXT = 0xd5;
	public const int EM_GETLINE = 0xc4;
	public const int EM_GETLINECOUNT = 0xba;
	public const int EM_GETMARGINS = 0xd4;
	public const int EM_GETPASSWORDCHAR = 0xd2;
	public const int EM_GETSCROLLPOS = 0x4dd;
	public const int EM_GETSEL = 0xb0;
	public const int EM_LIMITTEXT = 0xc5;
	public const int EM_LINEFROMCHAR = 0xc9;
	public const int EM_LINEINDEX = 0xbb;
	public const int EM_LINELENGTH = 0xc1;
	public const int EM_LINESCROLL = 0xb6;
	public const int EM_POSFROMCHAR = 0xd6;
	public const int EM_REPLACESEL = 0xc2;
	public const int EM_SCROLLCARET = 0xb7;
	public const int EM_SETBKGNDCOLOR = 0x443;
	public const int EM_SETLIMITTEXT = 0xc5;
	public const int EM_SETOPTIONS = 0x44d;
	public const int EM_SETPARAFORMAT = 0x447;
	public const int EM_SETPASSWORDCHAR = 0xcc;
	public const int EM_SETREADONLY = 0xcf;
	public const int EM_SETSEL = 0xb1;
	public const int EM_SETTABSTOPS = 0xcb;
	public const int EN_ALIGN_LTR_EC = 0x0700;
	public const int EN_ALIGN_RTL_EC = 0x0701;
	public const int EN_CHANGE = 0x300;
	public const int ERROR_NO_MORE_ITEMS = 0x103;
	public const int ESB_DISABLE_BOTH = 0x3;
	public const int ESB_ENABLE_BOTH = 0x0;
	public const int ES_AUTOHSCROLL = 0x80;
	public const int ES_CENTER = 0x1;
	public const int ES_MULTILINE = 0x4;
	public const int ES_NOHIDESEL = 0x100;
	public const int ES_PASSWORD = 0x20;
	public const int ES_READONLY = 0x800;
	public const int ES_RIGHT = 0x2;
	public const int FALT = 0x10;
	public const int FCONTROL = 0x8;
	public const int FNERR_INVALIDFILENAME = 0x3002;
	public const int FSHIFT = 0x4;
	public const int FVIRTKEY = 0x1;
	public const int GCS_COMPSTR = 0x8;
	public const int GCS_RESULTSTR = 0x800;
	public const int GMDI_USEDISABLED = 0x1;
	public const int GMEM_FIXED = 0x0;
	public const int GMEM_ZEROINIT = 0x40;
	public const int GN_CONTEXTMENU = 1000;
	public const int GPTR = 0x40;
	public const int GRADIENT_FILL_RECT_H = 0x0;
	public const int GRADIENT_FILL_RECT_V = 0x1;
	public const int GTL_NUMBYTES = 0x10;
	public const int GTL_NUMCHARS = 0x8;
	public const int GTL_PRECISE = 0x2;
	public const int GT_DEFAULT = 0x0;
	public const int GWL_EXSTYLE = cast(int)0xffffffec;
	public const int GWL_ID = -12;
	public const int GWL_STYLE = cast(int)0xfffffff0;
	public const int GWL_USERDATA = cast(int)0xffffffeb;
	public const int GWL_WNDPROC = cast(int)0xfffffffc;
	public const int GW_CHILD = 0x5;
	public const int GW_HWNDFIRST = 0x0;
	public const int GW_HWNDLAST = 0x1;
	public const int GW_HWNDNEXT = 0x2;
	public const int GW_HWNDPREV = 0x3;
	public const int HBMMENU_CALLBACK = cast(int)0xffffffff;
	public const int HDI_WIDTH = 0x1;
	public const int HDM_FIRST = 0x1200;
	public const int HDM_GETBITMAPMARGIN = HDM_FIRST + 21;
	public const int HDM_GETITEMCOUNT = 0x1200;
	public const int HDN_FIRST = cast(int)0xfffffed4;
	public const int HDN_BEGINTRACK = IsUnicode ? cast(int)0xfffffeba : cast(int)0xfffffece;
	public const int HDN_BEGINTRACKW = cast(int)0xfffffeba;
	public const int HDN_BEGINTRACKA = cast(int)0xfffffece;
	public const int HDN_DIVIDERDBLCLICKA = HDN_FIRST - 5;
	public const int HDN_DIVIDERDBLCLICKW = HDN_FIRST - 25;
	public const int HDN_DIVIDERDBLCLICK = IsUnicode ? HDN_DIVIDERDBLCLICKW : HDN_DIVIDERDBLCLICKA;
	public const int HDN_ITEMCHANGED = IsUnicode ? cast(int)0xfffffebf : cast(int)0xfffffed3;
	public const int HDN_ITEMCHANGEDW = cast(int)0xfffffebf;
	public const int HDN_ITEMCHANGEDA = cast(int)0xfffffed3;
	public const int HDN_ITEMDBLCLICKW = HDN_FIRST - 23;
	public const int HDN_ITEMDBLCLICKA = HDN_FIRST - 3;
	public const int HDN_ITEMDBLCLICK = IsUnicode ? HDN_ITEMDBLCLICKW : HDN_ITEMDBLCLICKA;
	public const int HEAP_ZERO_MEMORY = 0x8;
	public const int HELPINFO_MENUITEM = 0x2;
	public const int HINST_COMMCTRL = cast(int)0xffffffff;
	public const int HKEY_CLASSES_ROOT = cast(int)0x80000000;
	public const int HKEY_CURRENT_USER = cast(int)0x80000001;
	public const int HKEY_LOCAL_MACHINE = cast(int)0x80000002;
	public const int HORZRES = 0x8;
	public const int HTBORDER = 0x12;
	public const int HTCAPTION = 0x2;
	public const int HTCLIENT = 0x1;
	public const int HTERROR = -2;
	public const int HTHSCROLL = 0x6;
	public const int HTMENU = 0x5;
	public const int HTNOWHERE = 0x0;
	public const int HTSYSMENU = 0x3;
	public const int HTTRANSPARENT = cast(int)0xffffffff;
	public const int HTVSCROLL = 0x7;
	public const int HWND_BOTTOM = 0x1;
	public const int HWND_TOP = 0x0;
	public const int HWND_TOPMOST = cast(int)0xffffffff;
	public const int HWND_NOTOPMOST = -2;
	public const int ICC_COOL_CLASSES = 0x400;
	public const int ICM_NOTOPEN = 0x0;
	public const int ICON_BIG = 0x1;
	public const int ICON_SMALL = 0x0;
	public const int I_IMAGENONE = -2;
	public const int IDABORT = 0x3;
	public const int IDB_STD_SMALL_COLOR = 0x0;
	public const int IDC_APPSTARTING = 0x7f8a;
	public const int IDC_ARROW = 0x7f00;
	public const int IDC_CROSS = 0x7f03;
	public const int IDC_HAND = 0x7f89;
	public const int IDC_HELP = 0x7f8b;
	public const int IDC_IBEAM = 0x7f01;
	public const int IDC_NO = 0x7f88;
	public const int IDC_SIZE = 0x7f80;
	public const int IDC_SIZEALL = 0x7f86;
	public const int IDC_SIZENESW = 0x7f83;
	public const int IDC_SIZENS = 0x7f85;
	public const int IDC_SIZENWSE = 0x7f82;
	public const int IDC_SIZEWE = 0x7f84;
	public const int IDC_UPARROW = 0x7f04;
	public const int IDC_WAIT = 0x7f02;
	public const int IDI_WINLOGO = 0x7f05;
	public const int IDNO = 0x7;
	public const int IDOK = 0x1;
	public const int IDRETRY = 0x4;
	public const int IDYES = 0x6;
	public const int ILC_COLOR = 0x0;
	public const int ILC_COLOR16 = 0x10;
	public const int ILC_COLOR24 = 0x18;
	public const int ILC_COLOR32 = 0x20;
	public const int ILC_COLOR4 = 0x4;
	public const int ILC_COLOR8 = 0x8;
	public const int ILC_MASK = 0x1;
	public const int ILD_NORMAL = 0x0;
	public const int IMAGE_BITMAP = 0x0;
	public const int IMAGE_CURSOR = 0x2;
	public const int IMAGE_ICON = 0x1;
	public const int IME_CMODE_FULLSHAPE = 0x8;
	public const int IME_CMODE_KATAKANA = 0x2;
	public const int IME_CMODE_NATIVE = 0x1;
	public const int IME_CMODE_ROMAN = 0x10;
	public const int INFINITE = cast(int)0xffffffff;
	public const int KEY_ENUMERATE_SUB_KEYS = 0x8;
	public const int KEY_NOTIFY = 0x10;
	public const int KEY_QUERY_VALUE = 0x1;
	public const int KEY_READ = 0x20019;
	public const int LAYOUT_RTL = 0x1;
	public const int LBN_DBLCLK = 0x2;
	public const int LBN_SELCHANGE = 0x1;
	public const int LBS_EXTENDEDSEL = 0x800;
	public const int LBS_MULTIPLESEL = 0x8;
	public const int LBS_NOINTEGRALHEIGHT = 0x100;
	public const int LBS_NOTIFY = 0x1;
	public const int LB_ADDSTRING = 0x180;
	public const int LB_DELETESTRING = 0x182;
	public const int LB_ERR = cast(int)0xffffffff;
	public const int LB_ERRSPACE = cast(int)0xfffffffe;
	public const int LB_FINDSTRINGEXACT = 0x1a2;
	public const int LB_GETCARETINDEX = 0x19f;
	public const int LB_GETCOUNT = 0x18b;
	public const int LB_GETCURSEL = 0x188;
	public const int LB_GETHORIZONTALEXTENT = 0x193;
	public const int LB_GETITEMHEIGHT = 0x1a1;
	public const int LB_GETITEMRECT = 0x198;
	public const int LB_GETSEL = 0x187;
	public const int LB_GETSELCOUNT = 0x190;
	public const int LB_GETSELITEMS = 0x191;
	public const int LB_GETTEXT = 0x189;
	public const int LB_GETTEXTLEN = 0x18a;
	public const int LB_GETTOPINDEX = 0x18e;
	public const int LB_INITSTORAGE = 0x1a8;
	public const int LB_INSERTSTRING = 0x181;
	public const int LB_RESETCONTENT = 0x184;
	public const int LB_SELITEMRANGE = 0x19b;
	public const int LB_SELITEMRANGEEX = 0x183;
	public const int LB_SETCARETINDEX = 0x19e;
	public const int LB_SETCURSEL = 0x186;
	public const int LB_SETHORIZONTALEXTENT = 0x194;
	public const int LB_SETSEL = 0x185;
	public const int LB_SETTOPINDEX = 0x197;
	public const int LF_FACESIZE = 32;
	public const int LGRPID_ARABIC = 0xd;
	public const int LGRPID_HEBREW = 0xc;
	public const int LGRPID_INSTALLED = 1;
	public const int LCID_SUPPORTED = 0x2;
	public const int LOCALE_IDEFAULTANSICODEPAGE = 0x1004;
	public const int LOCALE_SISO3166CTRYNAME = 0x5a;
	public const int LOCALE_SISO639LANGNAME = 0x59;
	public const int LOGPIXELSX = 0x58;
	public const int LOGPIXELSY = 0x5a;
	public const int LPSTR_TEXTCALLBACK = cast(int)0xffffffff;
	public const int LR_DEFAULTCOLOR = 0x0;
	public const int LVCFMT_CENTER = 0x2;
	public const int LVCFMT_IMAGE = 0x800;
	public const int LVCFMT_LEFT = 0x0;
	public const int LVCFMT_RIGHT = 0x1;
	public const int LVCF_FMT = 0x1;
	public const int LVCF_IMAGE = 0x10;
	public const int LVCFMT_JUSTIFYMASK = 0x3;
	public const int LVCF_TEXT = 0x4;
	public const int LVCF_WIDTH = 0x2;
	public const int LVHT_ONITEM = 0xe;
	public const int LVHT_ONITEMICON = 0x2;
	public const int LVHT_ONITEMLABEL = 0x4;
	public const int LVHT_ONITEMSTATEICON = 0x8;
	public const int LVIF_IMAGE = 0x2;
	public const int LVIF_INDENT = 0x10;
	public const int LVIF_STATE = 0x8;
	public const int LVIF_TEXT = 0x1;
	public const int LVIR_BOUNDS = 0x0;
	public const int LVIR_ICON = 0x1;
	public const int LVIR_LABEL = 0x2;
	public const int LVIS_DROPHILITED = 0x8;
	public const int LVIS_FOCUSED = 0x1;
	public const int LVIS_SELECTED = 0x2;
	public const int LVIS_STATEIMAGEMASK = 0xf000;
	public const int LVM_APPROXIMATEVIEWRECT = 0x1040;
	public const int LVM_DELETEALLITEMS = 0x1009;
	public const int LVM_DELETECOLUMN = 0x101c;
	public const int LVM_DELETEITEM = 0x1008;
	public const int LVM_ENSUREVISIBLE = 0x1013;
	public const int LVM_FIRST = 0x1000;
	public const int LVM_GETBKCOLOR = 0x1000;
	public const int LVM_GETCOLUMN = IsUnicode ? 0x105f : 0x1019;
	public const int LVM_GETCOLUMNWIDTH = 0x101d;
	public const int LVM_GETCOUNTPERPAGE = 0x1028;
	public const int LVM_GETEXTENDEDLISTVIEWSTYLE = 0x1037;
	public const int LVM_GETHEADER = 0x101f;
	public const int LVM_GETIMAGELIST = 0x1002;
	public const int LVM_GETITEM = IsUnicode ? 0x104b : 0x1005;
	public const int LVM_GETITEMW = 0x104b;
	public const int LVM_GETITEMA = 0x1005;
	public const int LVM_GETITEMCOUNT = 0x1004;
	public const int LVM_GETITEMRECT = 0x100e;
	public const int LVM_GETITEMSTATE = 0x102c;
	public const int LVM_GETNEXTITEM = 0x100c;
	public const int LVM_GETSELECTEDCOUNT = 0x1032;
	public const int LVM_GETSTRINGWIDTH = IsUnicode ? 0x1057 : 0x1011;
	public const int LVM_GETSUBITEMRECT = 0x1038;
	public const int LVM_GETTEXTCOLOR = 0x1023;
	public const int LVM_GETTOOLTIPS = 0x104e;
	public const int LVM_GETTOPINDEX = 0x1027;
	public const int LVM_HITTEST = 0x1012;
	public const int LVM_INSERTCOLUMN = IsUnicode ? 0x1061 : 0x101b;
	public const int LVM_INSERTITEM = IsUnicode ? 0x104d : 0x1007;
	public const int LVM_SCROLL = 0x1014;
	public const int LVM_SETBKCOLOR = 0x1001;
	public const int LVM_SETCALLBACKMASK = LVM_FIRST + 11;
	public const int LVM_SETCOLUMN = IsUnicode ? 0x1060 : 0x101a;
	public const int LVM_SETCOLUMNWIDTH = 0x101e;
	public const int LVM_SETEXTENDEDLISTVIEWSTYLE = 0x1036;
	public const int LVM_SETIMAGELIST = 0x1003;
	public const int LVM_SETITEM = IsUnicode ? 0x104c : 0x1006;
	public const int LVM_SETITEMCOUNT = LVM_FIRST + 47;
	public const int LVM_SETITEMSTATE = 0x102b;
	public const int LVM_SETTEXTBKCOLOR = 0x1026;
	public const int LVM_SETTEXTCOLOR = 0x1024;
	public const int LVNI_FOCUSED = 0x1;
	public const int LVNI_SELECTED = 0x2;
	public const int LVN_BEGINDRAG = cast(int)0xffffff93;
	public const int LVN_BEGINRDRAG = cast(int)0xffffff91;
	public const int LVN_COLUMNCLICK = cast(int)0xffffff94;
	public const int LVN_FIRST = cast(int)0xffffff9c;
	public const int LVN_GETDISPINFOA = LVN_FIRST - 50;
	public const int LVN_GETDISPINFOW = LVN_FIRST - 77;
	public const int LVN_ITEMACTIVATE = cast(int)0xffffff8e;
	public const int LVN_ITEMCHANGED = cast(int)0xffffff9b;
	public const int LVN_MARQUEEBEGIN = cast(int)0xffffff64;
	public const int LVSCW_AUTOSIZE = cast(int)0xffffffff;
	public const int LVSCW_AUTOSIZE_USEHEADER = cast(int)0xfffffffe;
	public const int LVSICF_NOINVALIDATEALL = 0x1;
	public const int LVSICF_NOSCROLL = 0x2;
	public const int LVSIL_SMALL = 0x1;
	public const int LVSIL_STATE = 0x2;
	public const int LVS_EX_FULLROWSELECT = 0x20;
	public const int LVS_EX_GRIDLINES = 0x1;
	public const int LVS_EX_LABELTIP = 0x4000;
	public const int LVS_EX_ONECLICKACTIVATE = 0x40;
	public const int LVS_EX_SUBITEMIMAGES = 0x2;
	public const int LVS_EX_TRACKSELECT = 0x8;
	public const int LVS_EX_TWOCLICKACTIVATE = 0x80;
	public const int LVS_NOCOLUMNHEADER = 0x4000;
	public const int LVS_NOSCROLL = 0x2000;
	public const int LVS_OWNERDATA = 0x1000;
	public const int LVS_REPORT = 0x1;
	public const int LVS_SHAREIMAGELISTS = 0x40;
	public const int LVS_SHOWSELALWAYS = 0x8;
	public const int LVS_SINGLESEL = 0x4;
	public const int MAX_PATH = 260;
	public const int MA_NOACTIVATE = 0x3;
	public const int MB_ABORTRETRYIGNORE = 0x2;
	public const int MB_APPLMODAL = 0x0;
	public const int MB_ICONERROR = 0x10;
	public const int MB_ICONINFORMATION = 0x40;
	public const int MB_ICONQUESTION = 0x20;
	public const int MB_ICONWARNING = 0x30;
	public const int MB_OK = 0x0;
	public const int MB_OKCANCEL = 0x1;
	public const int MB_PRECOMPOSED = 0x1;
	public const int MB_RETRYCANCEL = 0x5;
	public const int MB_RTLREADING = 0x100000;
	public const int MB_SYSTEMMODAL = 0x1000;
	public const int MB_TASKMODAL = 0x2000;
	public const int MB_YESNO = 0x4;
	public const int MB_YESNOCANCEL = 0x3;
	public const int MDIS_ALLCHILDSTYLES = 0x0001;
	public const int MFS_CHECKED = 0x8;
	public const int MFS_DISABLED = 0x3;
	public const int MFS_GRAYED = 0x3;
	public const int MFT_RADIOCHECK = 0x200;
	public const int MFT_RIGHTJUSTIFY = 0x4000;
	public const int MFT_RIGHTORDER = 0x2000;
	public const int MFT_SEPARATOR = 0x800;
	public const int MFT_STRING = 0x0;
	public const int MF_BYCOMMAND = 0x0;
	public const int MF_BYPOSITION = 0x400;
	public const int MF_CHECKED = 0x8;
	public const int MF_DISABLED = 0x2;
	public const int MF_ENABLED = 0x0;
	public const int MF_GRAYED = 0x1;
	public const int MF_HILITE = 0x80;
	public const int MF_POPUP = 0x10;
	public const int MF_SEPARATOR = 0x800;
	public const int MF_SYSMENU = 0x2000;
	public const int MF_UNCHECKED = 0x0;
	public const int MIIM_BITMAP = 0x80;
	public const int MIIM_DATA = 0x20;
	public const int MIIM_ID = 0x2;
	public const int MIIM_STATE = 0x1;
	public const int MIIM_SUBMENU = 0x4;
	public const int MIIM_TYPE = 0x10;
	public const int MIM_STYLE = 0x10;
	public const int MK_CONTROL = 0x8;
	public const int MK_LBUTTON = 0x1;
	public const int MK_MBUTTON = 0x10;
	public const int MK_RBUTTON = 0x2;
	public const int MK_SHIFT = 0x4;
	public const int MM_TEXT = 0x1;
	public const int MNC_CLOSE = 0x1;
	public const int MNS_CHECKORBMP = 0x4000000;
	public const int MONITOR_DEFAULTTONEAREST = 0x2;
	public const int MONITORINFOF_PRIMARY = 0x1;
	public const int MWMO_INPUTAVAILABLE = 0x4;
	public const int NM_FIRST = 0x0;
	public const int NM_CLICK = cast(int)0xfffffffe;
	public const int NM_CUSTOMDRAW = NM_FIRST - 12;
	public const int NM_DBLCLK = cast(int)0xfffffffd;
	public const int NM_RECOGNIZEGESTURE = NM_FIRST - 16;
	public const int NM_RETURN = cast(int)0xfffffffc;
	public const int NOTSRCCOPY = 0x330008;
	public const int NULLREGION = 0x1;
	public const int NULL_BRUSH = 0x5;
	public const int NULL_PEN = 0x8;
	public const int NUMRESERVED = 106;
	public const int OBJID_CLIENT = cast(int)0xFFFFFFFC;
	public const int OBJ_FONT = 0x6;
	public const int OBJ_PEN = 0x1;
	public const int OBM_CHECKBOXES = 0x7ff7;
	public const int ODS_SELECTED = 0x1;
	public const int ODT_MENU = 0x1;
	public const int OFN_ALLOWMULTISELECT = 0x200;
	public const int OFN_EXPLORER = 0x80000;
	public const int OFN_HIDEREADONLY = 0x4;
	public const int OFN_NOCHANGEDIR = 0x8;
	public const int OPAQUE = 0x2;
	public const int PATCOPY = 0xf00021;
	public const int PATINVERT = 0x5a0049;
	public const int PBM_GETPOS = 0x408;
	public const int PBM_GETRANGE = 0x407;
	public const int PBM_SETBARCOLOR = 0x409;
	public const int PBM_SETBKCOLOR = 0x2001;
	public const int PBM_SETPOS = 0x402;
	public const int PBM_SETRANGE32 = 0x406;
	public const int PBM_STEPIT = 0x405;
	public const int PBS_SMOOTH = 0x1;
	public const int PBS_VERTICAL = 0x4;
	public const int PD_ALLPAGES = 0x0;
	public const int PD_COLLATE = 0x10;
	public const int PD_PAGENUMS = 0x2;
	public const int PD_PRINTTOFILE = 0x20;
	public const int PD_RETURNDC = 0x100;
	public const int PD_SELECTION = 0x1;
	public const int PD_USEDEVMODECOPIESANDCOLLATE = 0x40000;
	public const int PFM_TABSTOPS = 0x10;
	public const int PHYSICALHEIGHT = 0x6f;
	public const int PHYSICALOFFSETX = 0x70;
	public const int PHYSICALOFFSETY = 0x71;
	public const int PHYSICALWIDTH = 0x6e;
	public const int PLANES = 0xe;
	public const int PM_NOREMOVE = 0x0;
	public const int PM_REMOVE = 0x1;
	public const char[] PROGRESS_CLASS = "msctls_progress32"; //$NON-NLS-1$
	public const int PS_DASH = 0x1;
	public const int PS_DASHDOT = 0x3;
	public const int PS_DASHDOTDOT = 0x4;
	public const int PS_DOT = 0x2;
	public const int PS_ENDCAP_FLAT = 0x200;
	public const int PS_GEOMETRIC = 0x10000;
	public const int PS_JOIN_MITER = 0x2000;
	public const int PS_SOLID = 0x0;
	public const int PS_STYLE_MASK = 0xf;
	public const int QS_KEY = 0x0001;
	public const int QS_MOUSEMOVE = 0x0002;
	public const int QS_MOUSEBUTTON = 0x0004;
	public const int QS_POSTMESSAGE = 0x0008;
	public const int QS_TIMER = 0x0010;
	public const int QS_PAINT = 0x0020;
	public const int QS_SENDMESSAGE = 0x0040;
	public const int QS_ALLINPUT = QS_MOUSEMOVE | QS_MOUSEBUTTON | QS_KEY | QS_POSTMESSAGE | QS_TIMER | QS_PAINT | QS_SENDMESSAGE;
	public const int R2_COPYPEN = 0xd;
	public const int R2_XORPEN = 0x7;
	public const int RASTERCAPS = 0x26;
	public const int RASTER_FONTTYPE = 0x1;
	public const int RBBIM_CHILD = 0x10;
	public const int RBBIM_CHILDSIZE = 0x20;
	public const int RBBIM_COLORS = 0x2;
	public const int RBBIM_HEADERSIZE = 0x800;
	public const int RBBIM_ID = 0x100;
	public const int RBBIM_IDEALSIZE = 0x200;
	public const int RBBIM_SIZE = 0x40;
	public const int RBBIM_STYLE = 0x1;
	public const int RBBIM_TEXT = 0x4;
	public const int RBBS_BREAK = 0x1;
	public const int RBBS_GRIPPERALWAYS = 0x80;
	public const int RBBS_NOGRIPPER = 0x00000100;
	public const int RBBS_USECHEVRON = 0x00000200;
	public const int RBBS_VARIABLEHEIGHT = 0x40;
	public const int RBN_FIRST = cast(int)0xfffffcc1;
	public const int RBN_CHEVRONPUSHED = RBN_FIRST - 10;
	public const int RBN_HEIGHTCHANGE = cast(int)0xfffffcc1;
	public const int RBS_DBLCLKTOGGLE = 0x8000;
	public const int RBS_BANDBORDERS = 0x400;
	public const int RBS_VARHEIGHT = 0x200;
	public const int RB_DELETEBAND = 0x402;
	public const int RB_GETBANDBORDERS = 0x422;
	public const int RB_GETBANDCOUNT = 0x40c;
	public const int RB_GETBANDINFO = IsUnicode ? 0x41c : 0x41d;
	public const int RB_GETBARHEIGHT = 0x41b;
	public const int RB_GETBKCOLOR = 0x414;
	public const int RB_GETRECT = 0x409;
	public const int RB_GETTEXTCOLOR = 0x416;
	public const int RB_IDTOINDEX = 0x410;
	public const int RB_INSERTBAND = IsUnicode ? 0x40a : 0x401;
	public const int RB_MOVEBAND = 0x427;
	public const int RB_SETBANDINFO = IsUnicode ? 0x40b : 0x406;
	public const int RB_SETBKCOLOR = 0x413;
	public const int RB_SETTEXTCOLOR = 0x415;
	public const int RC_BITBLT = 0x1;
	public const int RC_PALETTE = 0x100;
	public const int RDW_ALLCHILDREN = 0x80;
	public const int RDW_ERASE = 0x4;
	public const int RDW_FRAME = 0x400;
	public const int RDW_INVALIDATE = 0x1;
	public const int RDW_UPDATENOW = 0x100;
	public const int READ_CONTROL = 0x20000;
	public const char[] REBARCLASSNAME = "ReBarWindow32"; //$NON-NLS-1$
	public const int RGN_AND = 0x1;
	public const int RGN_DIFF = 0x4;
	public const int RGN_ERROR = 0;
	public const int RGN_OR = 0x2;
	public const int SBS_HORZ = 0x0;
	public const int SBS_VERT = 0x1;
	public const int SB_BOTTOM = 0x7;
	public const int SB_CTL = 0x2;
	public const int SB_ENDSCROLL = 0x8;
	public const int SB_HORZ = 0x0;
	public const int SB_LINEDOWN = 0x1;
	public const int SB_LINEUP = 0x0;
	public const int SB_PAGEDOWN = 0x3;
	public const int SB_PAGEUP = 0x2;
	public const int SB_THUMBPOSITION = 0x4;
	public const int SB_THUMBTRACK = 0x5;
	public const int SB_TOP = 0x6;
	public const int SB_VERT = 0x1;
	public const int SCF_ALL = 0x4;
	public const int SCF_DEFAULT = 0x0;
	public const int SCF_SELECTION = 0x1;
	public const int SC_CLOSE = 0xf060;
	public const int SC_HSCROLL = 0xf080;
	public const int SC_KEYMENU = 0xf100;
	public const int SC_MAXIMIZE = 0xf030;
	public const int SC_MINIMIZE = 0xf020;
	public const int SC_NEXTWINDOW = 0xF040;
	public const int SC_RESTORE = 0xf120;
	public const int SC_SIZE = 0xf000;
	public const int SC_TASKLIST = 0xf130;
	public const int SC_VSCROLL = 0xf070;
	public const int SEM_FAILCRITICALERRORS = 0x1;
	public const int SF_RTF = 0x2;
	public const int SHCMBF_HIDDEN = 0x2;
	public const int SHCMBM_OVERRIDEKEY = 0x400 + 403;
	public const int SHCMBM_SETSUBMENU = 0x590;
	public const int SHCMBM_GETSUBMENU = 0x591;
	public const int SHMBOF_NODEFAULT = 0x1;
	public const int SHMBOF_NOTIFY = 0x2;
	public const int SHRG_RETURNCMD = 0x1;
	public const int SIF_ALL = 0x17;
	public const int SIF_DISABLENOSCROLL = 0x8;
	public const int SIF_PAGE = 0x2;
	public const int SIF_POS = 0x4;
	public const int SIF_RANGE = 0x1;
	public const int SIF_TRACKPOS = 0x10;
	public const int SIP_DOWN = 1;
	public const int SIP_UP = 0;
	public const int SIPF_ON = 0x1;
	public const int SIZE_MINIMIZED = 0x1;
	public const int SIZEPALETTE = 104;
	public const int SM_CMONITORS = 80;
	public const int SM_CXBORDER = 0x5;
	public const int SM_CXCURSOR = 0xd;
	public const int SM_CXEDGE = 0x2d;
	public const int SM_CXHSCROLL = 0x15;
	public const int SM_CXICON = 0x0b;
	public const int SM_CYICON = 0x0c;
	public const int SM_CXVIRTUALSCREEN = 78;
	public const int SM_CYVIRTUALSCREEN = 79;
	public const int SM_CXSMICON = 49;
	public const int SM_CYSMICON = 50;
	public const int SM_CXSCREEN = 0x0;
	public const int SM_XVIRTUALSCREEN = 76;
	public const int SM_YVIRTUALSCREEN = 77;
	public const int SM_CXVSCROLL = 0x2;
	public const int SM_CYBORDER = 0x6;
	public const int SM_CYCURSOR = 0xe;
	public const int SM_CYHSCROLL = 0x3;
	public const int SM_CYMENU = 0xf;
	public const int SM_CYSCREEN = 0x1;
	public const int SM_CYVSCROLL = 0x14;
	public const int SPI_GETWORKAREA = 0x30;
	public const int SPI_GETNONCLIENTMETRICS = 41;
	public const int SPI_GETWHEELSCROLLLINES = 104;
	public const int SPI_SETSIPINFO = 224;
	public const int SRCAND = 0x8800c6;
	public const int SRCCOPY = 0xcc0020;
	public const int SRCINVERT = 0x660046;
	public const int SRCPAINT = 0xee0086;
	public const int SS_BITMAP = 0xe;
	public const int SS_CENTER = 0x1;
	public const int SS_CENTERIMAGE = 0x200;
	public const int SS_ICON = 0x3;
	public const int SS_LEFT = 0x0;
	public const int SS_LEFTNOWORDWRAP = 0xc;
	public const int SS_NOTIFY = 0x100;
	public const int SS_OWNERDRAW = 0xd;
	public const int SS_REALSIZEIMAGE = 0x800;
	public const int SS_RIGHT = 0x2;
	public const int STANDARD_RIGHTS_READ = 0x20000;
	public const int STD_COPY = 0x1;
	public const int STD_CUT = 0x0;
	public const int STD_FILENEW = 0x6;
	public const int STD_FILEOPEN = 0x7;
	public const int STD_FILESAVE = 0x8;
	public const int STD_PASTE = 0x2;
	public const int STM_SETIMAGE = 0x172;
	public const int SWP_DRAWFRAME = 0x20;
	public const int SWP_NOACTIVATE = 0x10;
	public const int SWP_NOCOPYBITS = 0x100;
	public const int SWP_NOMOVE = 0x2;
	public const int SWP_NOREDRAW = 0x8;
	public const int SWP_NOSIZE = 0x1;
	public const int SWP_NOZORDER = 0x4;
	public const int SW_ERASE = 0x4;
	public const int SW_HIDE = 0x0;
	public const int SW_INVALIDATE = 0x2;
	public const int SW_MINIMIZE = 0x6;
	public const int SW_PARENTOPENING = 0x3;
	public const int SW_RESTORE = IsWinCE ? 0xd : 0x9;
	public const int SW_SCROLLCHILDREN = 0x1;
	public const int SW_SHOW = 0x5;
	public const int SW_SHOWMAXIMIZED = IsWinCE ? 0xb : 0x3;
	public const int SW_SHOWMINIMIZED = 0x2;
	public const int SW_SHOWMINNOACTIVE = 0x7;
	public const int SW_SHOWNA = 0x8;
	public const int SW_SHOWNOACTIVATE = 0x4;
	public const int SYNCHRONIZE = 0x100000;
	public const int SYSTEM_FONT = 0xd;
	public const int S_OK = 0x0;
	public const int TBIF_COMMAND = 0x20;
	public const int TBIF_STATE = 0x4;
	public const int TBIF_IMAGE = 0x1;
	public const int TBIF_LPARAM = 0x10;
	public const int TBIF_SIZE = 0x40;
	public const int TBIF_STYLE = 0x8;
	public const int TBIF_TEXT = 0x2;
	public const int TBM_GETLINESIZE = 0x418;
	public const int TBM_GETPAGESIZE = 0x416;
	public const int TBM_GETPOS = 0x400;
	public const int TBM_GETRANGEMAX = 0x402;
	public const int TBM_GETRANGEMIN = 0x401;
	public const int TBM_GETTHUMBRECT = 0x419;
	public const int TBM_SETLINESIZE = 0x417;
	public const int TBM_SETPAGESIZE = 0x415;
	public const int TBM_SETPOS = 0x405;
	public const int TBM_SETRANGEMAX = 0x408;
	public const int TBM_SETRANGEMIN = 0x407;
	public const int TBM_SETTICFREQ = 0x414;
	public const int TBN_DROPDOWN = cast(int)0xfffffd3a;
	public const int TBN_FIRST = cast(int)0xfffffd44;
	public const int TBSTATE_CHECKED = 0x1;
	public const int TBSTYLE_CUSTOMERASE = 0x2000;
	public const int TBSTYLE_DROPDOWN = 0x8;
	public const int TBSTATE_ENABLED = 0x4;
	public const int TBSTYLE_AUTOSIZE = 0x10;
	public const int TBSTYLE_EX_DRAWDDARROWS = 0x1;
	public const int TBSTYLE_FLAT = 0x800;
	public const int TBSTYLE_LIST = 0x1000;
	public const int TBSTYLE_TOOLTIPS = 0x100;
	public const int TBSTYLE_TRANSPARENT = 0x8000;
	public const int TBSTYLE_WRAPABLE = 0x200;
	public const int TBS_AUTOTICKS = 0x1;
	public const int TBS_BOTH = 0x8;
	public const int TBS_HORZ = 0x0;
	public const int TBS_VERT = 0x2;
	public const int TB_ADDSTRING = IsUnicode ? 0x44d : 0x41c;
	public const int TB_BUTTONCOUNT = 0x418;
	public const int TB_BUTTONSTRUCTSIZE = 0x41e;
	public const int TB_COMMANDTOINDEX = 0x419;
	public const int TB_DELETEBUTTON = 0x416;
	public const int TB_ENDTRACK = 0x8;
	public const int TB_GETBUTTON = 0x417;
	public const int TB_GETBUTTONINFO = IsUnicode ? 0x43f : 0x441;
	public const int TB_GETBUTTONSIZE = 0x43a;
	public const int TB_GETBUTTONTEXT = IsUnicode ? 0x44b : 0x42d;
	public const int TB_GETDISABLEDIMAGELIST = 0x437;
	public const int TB_GETHOTIMAGELIST = 0x435;
	public const int TB_GETHOTITEM = 0x0400 + 71;
	public const int TB_GETIMAGELIST = 0x431;
	public const int TB_GETITEMRECT = 0x41d;
	public const int TB_GETROWS = 0x428;
	public const int TB_GETSTATE = 0x412;
	public const int TB_GETTOOLTIPS = 0x423;
	public const int TB_INSERTBUTTON = IsUnicode ? 0x443 : 0x415;
	public const int TB_LOADIMAGES = 0x432;
	public const int TB_MAPACCELERATOR = 0x0400 + (IsUnicode ? 90 : 78);
	public const int TB_SETBITMAPSIZE = 0x420;
	public const int TB_SETBUTTONINFO = IsUnicode ? 0x440 : 0x442;
	public const int TB_SETBUTTONSIZE = 0x41f;
	public const int TB_SETDISABLEDIMAGELIST = 0x436;
	public const int TB_SETEXTENDEDSTYLE = 0x454;
	public const int TB_SETHOTIMAGELIST = 0x434;
	public const int TB_SETHOTITEM =  0x0400 + 72;
	public const int TB_SETIMAGELIST = 0x430;
	public const int TB_SETROWS = 0x427;
	public const int TB_SETSTATE = 0x411;
	public const int TCIF_IMAGE = 0x2;
	public const int TCIF_TEXT = 0x1;
	public const int TCI_SRCCHARSET = 0x1;
	public const int TCI_SRCCODEPAGE = 0x2;
	public const int TCM_ADJUSTRECT = 0x1328;
	public const int TCM_DELETEITEM = 0x1308;
	public const int TCM_GETCURSEL = 0x130b;
	public const int TCM_GETITEMCOUNT = 0x1304;
	public const int TCM_GETITEMRECT = 0x130a;
	public const int TCM_GETTOOLTIPS = 0x132d;
	public const int TCM_INSERTITEM = IsUnicode ? 0x133e : 0x1307;
	public const int TCM_SETCURSEL = 0x130c;
	public const int TCM_SETIMAGELIST = 0x1303;
	public const int TCM_SETITEM = IsUnicode ? 0x133d : 0x1306;
	public const int TCN_SELCHANGE = cast(int)0xfffffdd9;
	public const int TCN_SELCHANGING = cast(int)0xfffffdd8;
	public const int TCS_BOTTOM = 0x0002;
	public const int TCS_FOCUSNEVER = 0x8000;
	public const int TCS_MULTILINE = 0x200;
	public const int TCS_TABS = 0x0;
	public const int TCS_TOOLTIPS = 0x4000;
	public const int TME_HOVER = 0x1;
	public const int TME_LEAVE = 0x2;
	public const int TME_QUERY = 0x40000000;
	public const int TMPF_VECTOR = 0x2;
	public const char[] TOOLBARCLASSNAME = "ToolbarWindow32"; //$NON-NLS-1$
	public const char[] TOOLTIPS_CLASS = "tooltips_class32"; //$NON-NLS-1$
	public const int TPM_LEFTALIGN = 0x0;
	public const int TPM_LEFTBUTTON = 0x0;
	public const int TPM_RIGHTBUTTON = 0x2;
	public const int TPM_RIGHTALIGN = 0x8;
	public const char[] TRACKBAR_CLASS = "msctls_trackbar32"; //$NON-NLS-1$
	public const int TRANSPARENT = 0x1;
	public const int TTF_IDISHWND = 0x1;
	public const int TTF_SUBCLASS = 0x10;
	public const int TTF_RTLREADING = 0x4;
	public const int TTM_ADDTOOL = IsUnicode ? 0x432 : 0x404;
	public const int TTM_DELTOOL = IsUnicode ? 0x433 : 0x405;
	public const int TTM_SETMAXTIPWIDTH = 0x418;
	public const int TTM_UPDATE = 0x41D;
	public const int TTN_FIRST = cast(int)0xfffffdf8;
	public const int TTN_GETDISPINFO = IsUnicode ? cast(int)0xfffffdee : cast(int)0xfffffdf8;
	public const int TTN_GETDISPINFOW = cast(int)0xfffffdee;
	public const int TTN_GETDISPINFOA = cast(int)0xfffffdf8;
	public const int TTN_POP = TTN_FIRST - 2;
	public const int TTN_SHOW = TTN_FIRST - 1;
	public const int TTS_ALWAYSTIP = 0x1;
	public const int TV_FIRST = 0x1100;
	public const int TVE_COLLAPSE = 0x1;
	public const int TVE_EXPAND = 0x2;
	public const int TVGN_CARET = 0x9;
	public const int TVGN_CHILD = 0x4;
	public const int TVGN_DROPHILITED = 0x8;
	public const int TVGN_FIRSTVISIBLE = 0x5;
	public const int TVGN_LASTVISIBLE = 0xa;
	public const int TVGN_NEXT = 0x1;
	public const int TVGN_NEXTVISIBLE = 0x6;
	public const int TVGN_PARENT = 0x3;
	public const int TVGN_PREVIOUSVISIBLE = 0x7;
	public const int TVGN_ROOT = 0x0;
	public const int TVHT_ONITEM = 0x46;
	public const int TVHT_ONITEMICON = 0x2;
	public const int TVHT_ONITEMLABEL = 0x4;
	public const int TVHT_ONITEMSTATEICON = 0x40;
	public const int TVIF_HANDLE = 0x10;
	public const int TVIF_IMAGE = 0x2;
	public const int TVIF_PARAM = 0x4;
	public const int TVIF_SELECTEDIMAGE = 0x20;
	public const int TVIF_STATE = 0x8;
	public const int TVIF_TEXT = 0x1;
	public const int TVIS_EXPANDED = 0x20;
	public const int TVIS_SELECTED = 0x2;
	public const int TVIS_STATEIMAGEMASK = 0xf000;
	public const int TVI_FIRST = cast(int)0xffff0001;
	public const int TVI_LAST = cast(int)0xffff0002;
	public const int TVI_ROOT = cast(int)0xffff0000;
	public const int TVM_DELETEITEM = 0x1101;
	public const int TVM_ENSUREVISIBLE = 0x1114;
	public const int TVM_EXPAND = 0x1102;
	public const int TVM_GETBKCOLOR = 0x111f;
	public const int TVM_GETCOUNT = 0x1105;
	public const int TVM_GETIMAGELIST = 0x1108;
	public const int TVM_GETITEM = IsUnicode ? 0x113e : 0x110c;
	public const int TVM_GETITEMHEIGHT = 0x111c;
	public const int TVM_GETITEMRECT = 0x1104;
	public const int TVM_GETNEXTITEM = 0x110a;
	public const int TVM_GETTEXTCOLOR = 0x1120;
	public const int TVM_GETTOOLTIPS = TV_FIRST + 25;
	public const int TVM_GETVISIBLECOUNT = TV_FIRST + 16;
	public const int TVM_HITTEST = 0x1111;
	public const int TVM_INSERTITEM = IsUnicode ? 0x1132 : 0x1100;
	public const int TVM_SELECTITEM = 0x110b;
	public const int TVM_SETBKCOLOR = 0x111d;
	public const int TVM_SETIMAGELIST = 0x1109;
	public const int TVM_SETINSERTMARK = 0x111a;
	public const int TVM_SETITEM = IsUnicode ? 0x113f : 0x110d;
	public const int TVM_SETTEXTCOLOR = 0x111e;
	public const int TVN_BEGINDRAG = IsUnicode ? cast(int)0xfffffe38 : cast(int)0xfffffe69;
	public const int TVN_BEGINDRAGW = cast(int)0xfffffe38;
	public const int TVN_BEGINDRAGA = cast(int)0xfffffe69;
	public const int TVN_BEGINRDRAG = IsUnicode ? cast(int)0xfffffe37 : cast(int)0xfffffe68;
	public const int TVN_BEGINRDRAGW = cast(int)0xfffffe37;
	public const int TVN_BEGINRDRAGA = cast(int)0xfffffe68;
	public const int TVN_FIRST = cast(int)0xfffffe70;
	public const int TVN_ITEMEXPANDING = IsUnicode ? cast(int)0xfffffe3a : cast(int)0xfffffe6b;
	public const int TVN_ITEMEXPANDINGW = cast(int)0xfffffe3a;
	public const int TVN_ITEMEXPANDINGA = cast(int)0xfffffe6b;
	public const int TVN_SELCHANGED = IsUnicode ? cast(int)0xfffffe3d : cast(int)0xfffffe6e;
	public const int TVN_SELCHANGEDW = cast(int)0xfffffe3d;
	public const int TVN_SELCHANGEDA = cast(int)0xfffffe6e;
	public const int TVN_SELCHANGING = IsUnicode ? cast(int)0xfffffe3e : cast(int)0xfffffe6f;
	public const int TVN_SELCHANGINGW = cast(int)0xfffffe3e;
	public const int TVN_SELCHANGINGA = cast(int)0xfffffe6f;
	public const int TVSIL_NORMAL = 0x0;
	public const int TVSIL_STATE = 0x2;
	public const int TVS_DISABLEDRAGDROP = 0x10;
	public const int TVS_FULLROWSELECT = 0x1000;
	public const int TVS_HASBUTTONS = 0x1;
	public const int TVS_HASLINES = 0x2;
	public const int TVS_LINESATROOT = 0x4;
	public const int TVS_NOTOOLTIPS = 0x80;
	public const int TVS_SHOWSELALWAYS = 0x20;
	public const int UIS_INITIALIZE = 3;
	public const int VERTRES = 0xa;
	public const int VK_BACK = 0x8;
	public const int VK_CANCEL = 0x3;
	public const int VK_CAPITAL = 0x14;
	public const int VK_CONTROL = 0x11;
	public const int VK_DECIMAL = 0x6E;
	public const int VK_DELETE = 0x2e;
	public const int VK_DIVIDE = 0x6f;
	public const int VK_DOWN = 0x28;
	public const int VK_END = 0x23;
	public const int VK_ESCAPE = 0x1b;
	public const int VK_F1 = 0x70;
	public const int VK_F10 = 0x79;
	public const int VK_F11 = 0x7a;
	public const int VK_F12 = 0x7b;
	public const int VK_F13 = 0x7c;
	public const int VK_F14 = 0x7d;
	public const int VK_F15 = 0x7e;
	public const int VK_F2 = 0x71;
	public const int VK_F3 = 0x72;
	public const int VK_F4 = 0x73;
	public const int VK_F5 = 0x74;
	public const int VK_F6 = 0x75;
	public const int VK_F7 = 0x76;
	public const int VK_F8 = 0x77;
	public const int VK_F9 = 0x78;
	public const int VK_HOME = 0x24;
	public const int VK_INSERT = 0x2d;
	public const int VK_LBUTTON = 0x1;
	public const int VK_LEFT = 0x25;
	public const int VK_MBUTTON = 0x4;
	public const int VK_MENU = 0x12;
	public const int VK_MULTIPLY = 0x6A;
	public const int VK_NEXT = 0x22;
	public const int VK_NUMLOCK = 0x90;
	public const int VK_NUMPAD0 = 0x60;
	public const int VK_NUMPAD1 = 0x61;
	public const int VK_NUMPAD2 = 0x62;
	public const int VK_NUMPAD3 = 0x63;
	public const int VK_NUMPAD4 = 0x64;
	public const int VK_NUMPAD5 = 0x65;
	public const int VK_NUMPAD6 = 0x66;
	public const int VK_NUMPAD7 = 0x67;
	public const int VK_NUMPAD8 = 0x68;
	public const int VK_NUMPAD9 = 0x69;
	public const int VK_PAUSE = 0x13;
	public const int VK_PRIOR = 0x21;
	public const int VK_RBUTTON = 0x2;
	public const int VK_RETURN = 0xd;
	public const int VK_RIGHT = 0x27;
	public const int VK_SCROLL = 0x91;
	public const int VK_SEPARATOR = 0x6C;
	public const int VK_SHIFT = 0x10;
	public const int VK_SNAPSHOT = 0x2C;
	public const int VK_SPACE = 0x20;
	public const int VK_SUBTRACT = 0x6D;
	public const int VK_TAB = 0x9;
	public const int VK_UP = 0x26;
	public const int VK_ADD = 0x6B;
	public const int VK_APP1 = 0xc1;
	public const int VK_APP2 = 0xc2;
	public const int VK_APP3 = 0xc3;
	public const int VK_APP4 = 0xc4;
	public const int VK_APP5 = 0xc5;
	public const int VK_APP6 = 0xc6;
//	public const char[] WC_LISTVIEW = "SysListView32"; //$NON-NLS-1$
	public const wchar[] WC_LISTVIEW = "SysListView32"; //$NON-NLS-1$
//	public const char[] WC_TABCONTROL = "SysTabControl32"; //$NON-NLS-1$
	public const wchar[] WC_TABCONTROL = "SysTabControl32"; //$NON-NLS-1$
//	public const char[] WC_TREEVIEW = "SysTreeView32"; //$NON-NLS-1$
	public const wchar[] WC_TREEVIEW = "SysTreeView32"; //$NON-NLS-1$
	public const int WH_GETMESSAGE = 0x3;
	public const int WH_MSGFILTER = cast(int)0xFFFFFFFF;
	public const int WHEEL_DELTA = 120;
	public const int WHEEL_PAGESCROLL = cast(int)0xFFFFFFFF;
	public const int WM_ACTIVATE = 0x6;
	public const int WM_ACTIVATEAPP = 0x1c;
	public const int WM_APP = 0x8000;
	public const int WM_CANCELMODE = 0x1f;
	public const int WM_CHANGEUISTATE = 0x0127;
	public const int WM_CHAR = 0x102;
	public const int WM_CLEAR = 0x303;
	public const int WM_CLOSE = 0x10;
	public const int WM_COMMAND = 0x111;
	public const int WM_CONTEXTMENU = 0x7b;
	public const int WM_COPY = 0x301;
	public const int WM_CREATE = 0x0001;
	public const int WM_CTLCOLORBTN = 0x135;
	public const int WM_CTLCOLORDLG = 0x136;
	public const int WM_CTLCOLOREDIT = 0x133;
	public const int WM_CTLCOLORLISTBOX = 0x134;
	public const int WM_CTLCOLORMSGBOX = 0x132;
	public const int WM_CTLCOLORSCROLLBAR = 0x137;
	public const int WM_CTLCOLORSTATIC = 0x138;
	public const int WM_CUT = 0x300;
	public const int WM_DEADCHAR = 0x103;
	public const int WM_DESTROY = 0x2;
	public const int WM_DRAWITEM = 0x2b;
	public const int WM_ENDSESSION = 0x16;
	public const int WM_ENTERIDLE = 0x121;
	public const int WM_ERASEBKGND = 0x14;
	public const int WM_GETDLGCODE = 0x87;
	public const int WM_GETFONT = 0x31;
//	public const int WM_GETICON = 0x7f;
	public const int WM_GETOBJECT = 0x003D;
	public const int WM_HELP = 0x53;
	public const int WM_HOTKEY = 0x0312;
	public const int WM_HSCROLL = 0x114;
	public const int WM_IME_CHAR = 0x286;
	public const int WM_IME_COMPOSITION = 0x10f;
	public const int WM_INITDIALOG = 0x110;
	public const int WM_INITMENUPOPUP = 0x117;
	public const int WM_KEYDOWN = 0x100;
	public const int WM_KEYFIRST = 0x100;
	public const int WM_KEYLAST = 0x108;
	public const int WM_KEYUP = 0x101;
	public const int WM_KILLFOCUS = 0x8;
	public const int WM_LBUTTONDBLCLK = 0x203;
	public const int WM_LBUTTONDOWN = 0x201;
	public const int WM_LBUTTONUP = 0x202;
	public const int WM_MBUTTONDBLCLK = 0x209;
	public const int WM_MBUTTONDOWN = 0x207;
	public const int WM_MBUTTONUP = 0x208;
	public const int WM_MEASUREITEM = 0x2c;
	public const int WM_MENUCHAR = 0x120;
	public const int WM_MENUSELECT = 0x11f;
	public const int WM_MOUSEACTIVATE = 0x21;
	public const int WM_MOUSEFIRST = 0x200;
	public const int WM_MOUSEHOVER = 0x2a1;
	public const int WM_MOUSELEAVE = 0x2a3;
	public const int WM_MOUSEMOVE = 0x200;
	public const int WM_MOUSEWHEEL = 0x20a;
	public const int WM_MOUSELAST = 0x20d;
	public const int WM_MOVE = 0x3;
	public const int WM_NCACTIVATE = 0x86;
	public const int WM_NCCALCSIZE = 0x83;
	public const int WM_NCHITTEST = 0x84;
	public const int WM_NCLBUTTONDOWN = 0x00A1;
	public const int WM_NOTIFY = 0x4e;
	public const int WM_NULL = 0x0;
	public const int WM_PAINT = 0xf;
	public const int WM_PALETTECHANGED = 0x311;
	public const int WM_PARENTNOTIFY = 0x0210;
	public const int WM_PASTE = 0x302;
	public const int WM_PRINTCLIENT = 0x0318;
	public const int WM_QUERYENDSESSION = 0x11;
	public const int WM_QUERYNEWPALETTE = 0x30f;
	public const int WM_QUERYOPEN = 0x13;
	public const int WM_RBUTTONDBLCLK = 0x206;
	public const int WM_RBUTTONDOWN = 0x204;
	public const int WM_RBUTTONUP = 0x205;
	public const int WM_SETCURSOR = 0x20;
	public const int WM_SETFOCUS = 0x7;
	public const int WM_SETFONT = 0x30;
	public const int WM_SETICON = 0x80;
	public const int WM_SETREDRAW = 0xb;
	public const int WM_SETTINGCHANGE = 0x1A;
	public const int WM_SHOWWINDOW = 0x18;
	public const int WM_SIZE = 0x5;
	public const int WM_SYSCHAR = 0x106;
	public const int WM_SYSCOLORCHANGE = 0x15;
	public const int WM_SYSCOMMAND = 0x112;
	public const int WM_SYSKEYDOWN = 0x104;
	public const int WM_SYSKEYUP = 0x105;
	public const int WM_TIMER = 0x113;
	public const int WM_UNDO = 0x304;
	public const int WM_USER = 0x400;
	public const int WM_VSCROLL = 0x115;
	public const int WM_WINDOWPOSCHANGED = 0x47;
	public const int WM_WINDOWPOSCHANGING = 0x46;
	public const int WS_BORDER = 0x800000;
	public const int WS_CAPTION = 0xc00000;
	public const int WS_CHILD = 0x40000000;
	public const int WS_CLIPCHILDREN = 0x2000000;
	public const int WS_CLIPSIBLINGS = 0x4000000;
	public const int WS_EX_CAPTIONOKBTN = cast(int)0x80000000;
	public const int WS_EX_CLIENTEDGE = 0x200;
	public const int WS_EX_DLGMODALFRAME = 0x1;
	public const int WS_EX_LAYOUTRTL = 0x00400000;
	public const int WS_EX_LEFTSCROLLBAR = 0x00004000;
	public const int WS_EX_MDICHILD = 0x00000040;
	public const int WS_EX_NOINHERITLAYOUT = 0x00100000;
	public const int WS_EX_RIGHT = 0x00001000;
	public const int WS_EX_RTLREADING = 0x00002000;
	public const int WS_EX_STATICEDGE = 0x20000;
	public const int WS_EX_TOOLWINDOW = 0x80;
	public const int WS_EX_TOPMOST = 0x8;
	public const int WS_EX_TRANSPARENT = 0x20;
	public const int WS_HSCROLL = 0x100000;
	public const int WS_MAXIMIZEBOX = IsWinCE ? 0x20000 : 0x10000;
	public const int WS_MINIMIZEBOX = IsWinCE ? 0x10000 : 0x20000;
	public const int WS_OVERLAPPED = IsWinCE ? WS_BORDER | WS_CAPTION : 0x0;
	public const int WS_OVERLAPPEDWINDOW = 0xcf0000;
	public const int WS_POPUP = cast(int)0x80000000;
	public const int WS_SYSMENU = 0x80000;
	public const int WS_TABSTOP = 0x10000;
	public const int WS_THICKFRAME = 0x40000;
	public const int WS_VISIBLE = 0x10000000;
	public const int WS_VSCROLL = 0x200000;

/** Ansi/Unicode wrappers */

static int CallWindowProc (int lpPrevWndFunc, int hWnd, int Msg, int wParam, int lParam) {
	if (IsUnicode) return CallWindowProcW (lpPrevWndFunc, hWnd, Msg, wParam, lParam);
	return CallWindowProcA (lpPrevWndFunc, hWnd, Msg, wParam, lParam);
}

static short CharUpper (short ch) {
	if (IsUnicode) return CharUpperW (ch);
	return CharUpperA (ch);
}

static short CharLower (short ch) {
	if (IsUnicode) return CharLowerW (ch);
	return CharLowerA (ch);
}

/*bool ChooseColor (CHOOSECOLOR lpcc) {
	if (IsUnicode) return ChooseColorW (lpcc);
	return ChooseColorA (lpcc);
}

bool ChooseFont (CHOOSEFONT chooseFont) {
	if (IsUnicode) return ChooseFontW (chooseFont);
	return ChooseFontA (chooseFont);
}
*/
private static char* c_str(TCHAR str) {
	return (str is null) ? cast(char*)null : str.bytes;
}

private static wchar* w_str(TCHAR str) {
	return (str is null) ? cast(wchar*)null : str.chars;
}

static int CreateAcceleratorTable (byte[] lpaccl, int cEntries) {
	if (IsUnicode)
            return CreateAcceleratorTableW (cast(wchar*)&lpaccl[0], cEntries);

	return CreateAcceleratorTableA (cast(char*)&lpaccl[0], cEntries);
}

static int CreateDC (TCHAR lpszDriver, TCHAR lpszDevice, int lpszOutput, int lpInitData) {
	if (IsUnicode) {
		return CreateDCW (w_str(lpszDriver), w_str(lpszDevice), lpszOutput, lpInitData);
	} else {
		return CreateDCA (c_str(lpszDriver), c_str(lpszDevice), lpszOutput, lpInitData);
	}
}

static int CreateFontIndirect (int lplf) {
	if (IsUnicode)
		return CreateFontIndirectW (cast(LOGFONTW*)lplf);
	else
		return CreateFontIndirectA (cast(LOGFONTA*)lplf);
}

static int CreateFontIndirect (LOGFONT* lplf) {
	if (IsUnicode)
		return CreateFontIndirectW (cast(LOGFONTW*)lplf);
	else
		return CreateFontIndirectA (cast(LOGFONTA*)lplf);
}



static int CreateWindowEx (int dwExStyle, TCHAR lpClassName, TCHAR lpWindowName, int dwStyle, int X, int Y, int nWidth, int nHeight, int hWndParent, int hMenu, int hInstance, CREATESTRUCT* lpParam) {
	if (IsUnicode) {
		return CreateWindowExW (dwExStyle, w_str(lpClassName), w_str(lpWindowName), dwStyle, X, Y, nWidth, nHeight, hWndParent, hMenu, hInstance, lpParam);
	} else {
		return CreateWindowExA (dwExStyle, c_str(lpClassName), c_str(lpWindowName), dwStyle, X, Y, nWidth, nHeight, hWndParent, hMenu, hInstance, lpParam);
	}
}

static int DefMDIChildProc (int hWnd, int Msg, int wParam, int lParam) {
	if (IsUnicode) return DefMDIChildProcW (hWnd, Msg, wParam, lParam);
	return DefMDIChildProcA (hWnd, Msg, wParam, lParam);
}

static int DefFrameProc (int hWnd, int hWndMDIClient, int Msg, int wParam, int lParam) {
	if (IsUnicode) return DefFrameProcW (hWnd, hWndMDIClient, Msg, wParam, lParam);
	return DefFrameProcA (hWnd, hWndMDIClient, Msg, wParam, lParam);
}
static int DefWindowProc (int hWnd, int Msg, int wParam, int lParam) {
	if (IsUnicode) return DefWindowProcW (hWnd, Msg, wParam, lParam);
	return DefWindowProcA (hWnd, Msg, wParam, lParam);
}


static int DispatchMessage(MSG* lpmsg) {
	if (IsUnicode) return DispatchMessageW (lpmsg);
	return DispatchMessageA (lpmsg);
}

static int DragQueryFile (int hDrop, int iFile, TCHAR lpszFile, int cch) {
	if (IsUnicode) {
		wchar* lpszFile1 = w_str(lpszFile);
		return DragQueryFileW (hDrop, iFile, lpszFile1, cch);
	}
	char* lpszFile1 = c_str(lpszFile);
	return DragQueryFileA (hDrop, iFile, lpszFile1, cch);
}

static bool DrawState (int hdc, int hbr, int lpOutputFunc, int lData, int wData, int x, int y, int cx, int cy, int fuFlags) {
	if (IsUnicode) return DrawStateW (hdc, hbr, lpOutputFunc, lData, wData, x, y, cx, cy, fuFlags);
	return DrawStateA (hdc, hbr, lpOutputFunc, lData, wData, x, y, cx, cy, fuFlags);
}


static int DrawText (int hDC, TCHAR lpString, int nCount, RECT* lpRect, int uFormat) {
	if (IsUnicode) {
		wchar* lpString1 = w_str(lpString);
		return DrawTextW (hDC, lpString1, nCount, lpRect, uFormat);
	}
	char* lpString1 = c_str(lpString);
	return DrawTextA (hDC, lpString1, nCount, lpRect, uFormat);
}

static int EnumFontFamilies (int hdc, TCHAR lpszFamily, int lpEnumFontFamProc, int lParam) {
	if (IsUnicode) {
		wchar* lpszFamily1 = w_str(lpszFamily);
		return EnumFontFamiliesW (hdc, lpszFamily1, lpEnumFontFamProc, lParam);
	}
	char* lpszFamily1 = c_str(lpszFamily);
	return EnumFontFamiliesA (hdc, lpszFamily1, lpEnumFontFamProc, lParam);
}

static bool EnumSystemLocales (void* lpLocaleEnumProc, int dwFlags) {
	if (IsUnicode)
		return EnumSystemLocalesW (lpLocaleEnumProc, dwFlags);
	else
		return EnumSystemLocalesA (lpLocaleEnumProc, dwFlags);
}


static bool EnumSystemLanguageGroups (int pLangGroupEnumProc, int dwFlags, int lParam) {
	if (IsUnicode) return EnumSystemLanguageGroupsW (pLangGroupEnumProc, dwFlags, lParam);
	return EnumSystemLanguageGroupsA (pLangGroupEnumProc, dwFlags, lParam);
}

static int ExpandEnvironmentStrings (TCHAR lpSrc, TCHAR lpDst, int nSize) {
	if (IsUnicode) {
		wchar* lpSrc1 = w_str(lpSrc);
		wchar* lpDst1 = w_str(lpDst);
		return ExpandEnvironmentStringsW (lpSrc1, lpDst1, nSize);
	}
	char* lpSrc1 = c_str(lpSrc);
	char* lpDst1 = c_str(lpDst);
	return ExpandEnvironmentStringsA (lpSrc1, lpDst1, nSize);
}

static int ExtractIconEx (TCHAR lpszFile, int nIconIndex, int [] phiconLarge, int [] phiconSmall, int nIcons) {
	if (IsUnicode) {
		wchar* lpszFile1 = w_str(lpszFile);
		return ExtractIconExW (lpszFile1, nIconIndex, phiconLarge, phiconSmall, nIcons);
	}
	char* lpszFile1 = c_str(lpszFile);
	return ExtractIconExA (lpszFile1, nIconIndex, phiconLarge, phiconSmall, nIcons);
}

static bool ExtTextOut(int hdc, int X, int Y, int fuOptions, RECT* lprc, TCHAR lpString, int cbCount, int[] lpDx) {
	if (IsUnicode) {
		wchar* lpString1 = w_str(lpString);
		return ExtTextOutW (hdc, X, Y, fuOptions, lprc, lpString1, cbCount, lpDx);
	}
	char* lpString1 = c_str(lpString);
	return ExtTextOutA (hdc, X, Y, fuOptions, lprc, lpString1, cbCount, lpDx);
}

static int FindWindow (TCHAR lpClassName, TCHAR lpWindowName) {
	if (IsUnicode) {
		wchar* lpClassName1 = w_str(lpClassName);
		wchar* lpWindowName1 = w_str(lpWindowName);
		return FindWindowW (lpClassName1, lpWindowName1);
	}
	char* lpClassName1 = c_str(lpClassName);
	char* lpWindowName1 = c_str(lpWindowName);
	return FindWindowA (lpClassName1, lpWindowName1);
}

static bool GetCharABCWidths (int hdc, int iFirstChar, int iLastChar, int [] lpabc) {
	if (IsUnicode) return GetCharABCWidthsW (hdc,iFirstChar, iLastChar, lpabc);
	return GetCharABCWidthsA (hdc,iFirstChar, iLastChar, lpabc);
}


static int GetCharacterPlacement (int hdc, TCHAR lpString, int nCount, int nMaxExtent, GCP_RESULTS* lpResults, int dwFlags) {
	if (IsUnicode) {
		wchar* lpString1 = w_str(lpString);
		return GetCharacterPlacementW (hdc, lpString1, nCount, nMaxExtent, lpResults, dwFlags);
	}
	char* lpString1 = c_str(lpString);
	return GetCharacterPlacementA (hdc, lpString1, nCount, nMaxExtent, lpResults, dwFlags);
}

static bool GetCharWidth (int hdc, int iFirstChar, int iLastChar, int [] lpabc) {
	if (IsUnicode) return GetCharWidthW (hdc,iFirstChar, iLastChar, lpabc);
	return GetCharWidthA (hdc,iFirstChar, iLastChar, lpabc);
}

static bool GetClassInfo (int hInstance, TCHAR lpClassName, WNDCLASS* lpWndClass) {
	if (IsUnicode) {
		wchar* lpClassName1 = w_str(lpClassName);
		return GetClassInfoW (hInstance, lpClassName1, lpWndClass);
	}
        char* lpClassName1 = c_str(lpClassName);
	return GetClassInfoA (hInstance, lpClassName1, lpWndClass);
}



static int GetClipboardFormatName (int format, TCHAR lpszFormatName, int cchMaxCount) {
	if (IsUnicode) {
		wchar* lpszFormatName1 = w_str(lpszFormatName);
		return GetClipboardFormatNameW (format, lpszFormatName1, cchMaxCount);
	}
	char* lpszFormatName1 = c_str(lpszFormatName);
	return GetClipboardFormatNameA (format, lpszFormatName1, cchMaxCount);
}

static int GetKeyNameText (int lParam, TCHAR lpString, int nSize) {
	if (IsUnicode) {
		wchar* lpString1 = w_str(lpString);
		return GetKeyNameTextW (lParam, lpString1, nSize);
	}
	char* lpString1 = c_str(lpString);
	return GetKeyNameTextA (lParam, lpString1, nSize);
}

static int GetLocaleInfo (int Locale, int LCType, TCHAR lpLCData, int cchData) {
	if (IsUnicode) {
		wchar* lpLCData1 = w_str(lpLCData);
		return GetLocaleInfoW (Locale, LCType, lpLCData1, cchData);
	}
	char* lpLCData1 = c_str(lpLCData);
	return GetLocaleInfoA (Locale, LCType, lpLCData1, cchData);
}


static bool GetMenuItemInfo (int hMenu, int uItem, bool fByPosition, MENUITEMINFO* lpmii) {
	if (IsUnicode) return GetMenuItemInfoW (hMenu, uItem, fByPosition, lpmii);
	return GetMenuItemInfoA (hMenu, uItem, fByPosition, lpmii);
}

static bool GetMessage (MSG lpMsg, int hWnd, int wMsgFilterMin, int wMsgFilterMax) {
	if (IsUnicode) return GetMessageW (lpMsg, hWnd, wMsgFilterMin, wMsgFilterMax);
	return GetMessageA (lpMsg, hWnd, wMsgFilterMin, wMsgFilterMax);
}

static int GetModuleHandle (TCHAR lpModuleName) {
	if (IsUnicode) {
		wchar* lpModuleName1 = w_str(lpModuleName);
		return GetModuleHandleW (lpModuleName1);
	}
	char* lpModuleName1 = c_str(lpModuleName);
	return GetModuleHandleA (lpModuleName1);
}

static bool GetMonitorInfo (int hmonitor, MONITORINFO* lpmi) {
	if (IsUnicode) return GetMonitorInfoW (hmonitor, lpmi);
	return GetMonitorInfoA (hmonitor, lpmi);
}

static int GetObject (int hgdiobj, int cbBuffer, BITMAP* lpvObject) {
	if (IsUnicode) return GetObjectW (hgdiobj, cbBuffer, lpvObject);
	return GetObjectA (hgdiobj, cbBuffer, lpvObject);
}

static int GetObject (int hgdiobj, int cbBuffer, DIBSECTION* lpvObject) {
	if (IsUnicode) return GetObjectW (hgdiobj, cbBuffer, lpvObject);
	return GetObjectA (hgdiobj, cbBuffer, lpvObject);
}

static int GetObject (int hgdiobj, int cbBuffer, LOGBRUSH* lpvObject) {
	if (IsUnicode) return GetObjectW (hgdiobj, cbBuffer, lpvObject);
	return GetObjectA (hgdiobj, cbBuffer, lpvObject);
}


static int GetObject (int hgdiobj, int cbBuffer, LOGFONT* lpvObject) {
	if (IsUnicode)
		return GetObjectW (hgdiobj, cbBuffer, cast(LOGFONTW*)lpvObject);
	else
		return GetObjectA (hgdiobj, cbBuffer, cast(LOGFONTA*)lpvObject);
}

static int GetObject (int hgdiobj, int cbBuffer, LOGPEN* lpvObject) {
	if (IsUnicode) return GetObjectW (hgdiobj, cbBuffer, lpvObject);
	return GetObjectA (hgdiobj, cbBuffer, lpvObject);
}



static bool GetOpenFileName (OPENFILENAME* lpofn) {
	if (IsUnicode) return GetOpenFileNameW (lpofn);
	return GetOpenFileNameA (lpofn);
}

static int GetProfileString (TCHAR lpAppName, TCHAR lpKeyName, TCHAR lpDefault, TCHAR lpReturnedString, int nSize) {
	if (IsUnicode) {
		wchar* lpAppName1 = w_str(lpAppName);
		wchar* lpKeyName1 = w_str(lpKeyName);
		wchar* lpDefault1 = w_str(lpDefault);
		wchar* lpReturnedString1 = w_str(lpReturnedString);
		return GetProfileStringW (lpAppName1, lpKeyName1, lpDefault1, lpReturnedString1, nSize);
	}
	char* lpAppName1 = c_str(lpAppName);
	char* lpKeyName1 = c_str(lpKeyName);
	char* lpDefault1 = c_str(lpDefault);
	char* lpReturnedString1 = c_str(lpReturnedString);
	return GetProfileStringA (lpAppName1, lpKeyName1, lpDefault1, lpReturnedString1, nSize);
}

static bool GetSaveFileName (OPENFILENAME* lpofn) {
	if (IsUnicode) return GetSaveFileNameW (lpofn);
	return GetSaveFileNameA (lpofn);
}

static bool GetTextExtentPoint32 (int hdc, TCHAR lpString, int cbString, SIZE* lpSize) {
	if (IsUnicode) {
		wchar* lpString1 = w_str(lpString);
		return GetTextExtentPoint32W (hdc, lpString1, cbString, lpSize);
	}
	char* lpString1 = c_str(lpString);
	return GetTextExtentPoint32A (hdc, lpString1, cbString, lpSize);
}


static bool GetTextMetrics (int hdc, TEXTMETRIC* lptm) {
	if (IsUnicode) return GetTextMetricsW (hdc, cast(TEXTMETRICW*)lptm);
	return GetTextMetricsA (hdc, cast(TEXTMETRICA*)lptm);
}


static bool GetVersionEx (OSVERSIONINFO* lpVersionInfo) {
	if (IsUnicode)
		return GetVersionExW (cast(OSVERSIONINFOW*)lpVersionInfo);
	else
		return GetVersionExA (cast(OSVERSIONINFOA*)lpVersionInfo);
}

static int GetWindowLong (int hWnd, uint nIndex) {
	if (IsUnicode) return GetWindowLongW (hWnd, nIndex);
	return GetWindowLongA (hWnd, nIndex);
}

static int GetWindowText (int hWnd, TCHAR lpString, int nMaxCount) {
	if (IsUnicode) {
		wchar* lpString1 = w_str(lpString);
		return GetWindowTextW (hWnd, lpString1, nMaxCount);
	} else {
		char* lpString1 = lpString is null ? cast(char*)null : lpString.bytes;
		return GetWindowTextA (hWnd, lpString1, nMaxCount);
	}
}

static int GetWindowTextLength (int hWnd) {
	if (IsUnicode)
		return GetWindowTextLengthW (hWnd);
	else
		return GetWindowTextLengthA (hWnd);
}

static bool ImmGetCompositionFont (int hIMC, LOGFONT* lplf) {
	if (IsUnicode)
		return ImmGetCompositionFontW (hIMC, cast(LOGFONTW*)lplf);
	else
		return ImmGetCompositionFontA (hIMC, cast(LOGFONTA*)lplf);
}

static bool ImmSetCompositionFont (int hIMC, LOGFONT* lplf) {
	if (IsUnicode)
		return ImmSetCompositionFontW (hIMC, cast(LOGFONTW*)lplf);
	else
		return ImmSetCompositionFontA (hIMC, cast(LOGFONTA*)lplf);
}

static int ImmGetCompositionString (int hIMC, int dwIndex, TCHAR lpBuf, int dwBufLen) {
	if (IsUnicode) {
		wchar* lpBuf1 = w_str(lpBuf);
		return ImmGetCompositionStringW (hIMC, dwIndex, lpBuf1, dwBufLen);
	} else {
		char* lpBuf1 = lpBuf is null ? cast(char*)null : lpBuf.bytes;
		return ImmGetCompositionStringA (hIMC, dwIndex, lpBuf1, dwBufLen);
	}
}

static bool InsertMenu (int hMenu, int uPosition, int uFlags, int uIDNewItem, TCHAR lpNewItem) {
	if (IsUnicode) {
		wchar* lpNewItem1 = w_str(lpNewItem);
		return InsertMenuW (hMenu, uPosition, uFlags, uIDNewItem, lpNewItem1);
	} else {
		char* lpNewItem1 = lpNewItem is null ? cast(char*)null : lpNewItem.bytes;
		return InsertMenuA (hMenu, uPosition, uFlags, uIDNewItem, lpNewItem1);
	}
}

static bool InsertMenuItem (int hMenu, int uItem, bool fByPosition, MENUITEMINFO* lpmii) {
	if (IsUnicode) return InsertMenuItemW (hMenu, uItem, fByPosition, lpmii);
	return InsertMenuItemA (hMenu, uItem, fByPosition, lpmii);
}

static int LoadBitmap (int hInstance, int lpBitmapName) {
	if (IsUnicode) return LoadBitmapW (hInstance, lpBitmapName);
	return LoadBitmapA (hInstance, lpBitmapName);
}

static int LoadCursor (int hInstance, int lpCursorName) {
	if (IsUnicode) return LoadCursorW (hInstance, lpCursorName);
	return LoadCursorA (hInstance, lpCursorName);
}

static int LoadIcon (int hInstance, int lpIconName) {
	if (IsUnicode) return LoadIconW (hInstance, lpIconName);
	return LoadIconA (hInstance, lpIconName);
}

static int LoadImage (int hinst, TCHAR lpszName, int uType, int cxDesired, int cyDesired, int fuLoad) {
	if (IsUnicode) {
		wchar* lpszName1 = w_str(lpszName);
		return LoadImageW (hinst, lpszName1, uType, cxDesired, cyDesired, fuLoad);
	} else {
		char* lpszName1 = lpszName is null ? cast(char*)null : lpszName.bytes;
		return LoadImageA (hinst, lpszName1, uType, cxDesired, cyDesired, fuLoad);
	}
}

static int LoadLibrary (TCHAR lpLibFileName) {
	if (IsUnicode) {
		wchar* lpLibFileName1 = w_str(lpLibFileName);
		return LoadLibraryW (lpLibFileName1);
	} else {
		char* lpLibFileName1 = lpLibFileName is null ? cast(char*)null : lpLibFileName.bytes;
		return LoadLibraryA (lpLibFileName1);
	}
}

static int MapVirtualKey (int uCode, int uMapType) {
	if (IsUnicode) return MapVirtualKeyW (uCode, uMapType);
	return MapVirtualKeyA (uCode, uMapType);
}

static int MessageBox (int hWnd, TCHAR lpText, TCHAR lpCaption, int uType) {
	if (IsUnicode) {
		return MessageBoxW (hWnd, w_str(lpText), w_str(lpCaption), uType);
	}
	return MessageBoxA (hWnd, c_str(lpText), c_str(lpCaption), uType);
}

static void MoveMemory (void* Destination, TCHAR Source, int Length) {
//static void MoveMemory (int Destination, TCHAR Source, int Length) {
	if (IsUnicode) {
	    wchar* Source1 = w_str(Source);
            (cast(wchar*)Destination)[0..Length] = Source1[0..Length];
	    //MoveMemory(Destination, Source1, Length);
	} else {
            char* Source1 = c_str(Source);
            (cast(char*)Destination)[0..Length] = Source1[0..Length];
            //MoveMemory(Destination, Source1, Length);
	}
}

static void MoveMemory (TCHAR Destination, void* Source, int Length) {
	if (IsUnicode) {
		wchar* Destination1 = w_str(Destination);
		Destination1 = (cast(wchar*)Source)[0..Length];
	} else {
		char* Destination1 = c_str(Destination);
                Destination1 = (cast(char*)Source)[0..Length];
	}
}

static void MoveMemory (int Destination, LOGFONT* Source, int Length) {
    (cast(byte*)Destination)[0..Length] = (cast(byte*)Source)[0..Length];
}

static void MoveMemory (LOGFONT* Destination, int Source, int Length) {
    (cast(byte*)Destination)[0..Length] = (cast(byte*)Source)[0..Length];
}

static void MoveMemory (int Destination, NMTTDISPINFO* Source, int Length) {
    (cast(byte*)Destination)[0..Length] = (cast(byte*)Source)[0..Length];
}

static void MoveMemory (NMTTDISPINFO* Destination, int Source, int Length) {
    (cast(byte*)Destination)[0..Length] = (cast(byte*)Source)[0..Length];
}



static bool PeekMessage(MSG* lpMsg, int hWnd, int wMsgFilterMin, int wMsgFilterMax, int wRemoveMsg) {
	if (IsUnicode) return PeekMessageW (lpMsg, hWnd, wMsgFilterMin, wMsgFilterMax, wRemoveMsg);
	return PeekMessageA (lpMsg, hWnd, wMsgFilterMin, wMsgFilterMax, wRemoveMsg);
}

static bool PostMessage (int hWnd, int Msg, int wParam, int lParam) {
	if (IsUnicode) return PostMessageW (hWnd, Msg, wParam, lParam);
	return PostMessageA (hWnd, Msg, wParam, lParam);
}

static bool PostThreadMessage (int idThread, int Msg, int wParam, int lParam) {
	if (IsUnicode) return PostThreadMessageW (idThread, Msg, wParam, lParam);
	return PostThreadMessageA (idThread, Msg, wParam, lParam);
}

static bool PrintDlg (PRINTDLG lppd) {
	if (IsUnicode) return PrintDlgW (lppd);
	return PrintDlgA (lppd);
}

static int RegEnumKeyEx (int hKey, int dwIndex, TCHAR lpName, int [] lpcName, int [] lpReserved, TCHAR lpClass, int [] lpcClass, FILETIME* lpftLastWriteTime) {
	if (IsUnicode) {
		wchar* lpName1 = w_str(lpName);
		wchar* lpClass1 = w_str(lpClass);
		return RegEnumKeyExW (hKey, dwIndex, lpName1, lpcName, lpReserved, lpClass1, lpcClass, lpftLastWriteTime);
	}
	char* lpName1 = c_str(lpName);
	char* lpClass1 = c_str(lpClass);
	return RegEnumKeyExA (hKey, dwIndex, lpName1, lpcName, lpReserved, lpClass1, lpcClass, lpftLastWriteTime);
}

static int RegisterClass (WNDCLASS* lpWndClass) {
	if (IsUnicode) return RegisterClassW (lpWndClass);
	return RegisterClassA (lpWndClass);
}

static int RegisterClipboardFormat (TCHAR lpszFormat) {
	if (IsUnicode) {
		wchar* lpszFormat1 = w_str(lpszFormat);
		return RegisterClipboardFormatW (lpszFormat1);
	}
	char* lpszFormat1 = c_str(lpszFormat);
	return RegisterClipboardFormatA (lpszFormat1);
}

static int RegOpenKeyEx (int hKey, TCHAR lpSubKey, int ulOptions, int samDesired, int[] phkResult) {
	if (IsUnicode) {
		wchar* lpSubKey1 = w_str(lpSubKey);
		return RegOpenKeyExW (hKey, lpSubKey1, ulOptions, samDesired, phkResult);
	}
	char* lpSubKey1 = c_str(lpSubKey);
	return RegOpenKeyExA (hKey, lpSubKey1, ulOptions, samDesired, phkResult);
}

static int RegQueryInfoKey (int hKey, int lpClass, int[] lpcbClass, int lpReserved, int[] lpSubKeys, int[] lpcbMaxSubKeyLen, int[] lpcbMaxClassLen, int[] lpcValues, int[] lpcbMaxValueNameLen, int[] lpcbMaxValueLen, int[] lpcbSecurityDescriptor, int lpftLastWriteTime){
	if (IsUnicode) return RegQueryInfoKeyW (hKey, lpClass, lpcbClass, lpReserved, lpSubKeys, lpcbMaxSubKeyLen, lpcbMaxClassLen, lpcValues, lpcbMaxValueNameLen, lpcbMaxValueLen, lpcbSecurityDescriptor, lpftLastWriteTime);
	return RegQueryInfoKeyA (hKey, lpClass, lpcbClass, lpReserved, lpSubKeys, lpcbMaxSubKeyLen, lpcbMaxClassLen, lpcValues, lpcbMaxValueNameLen, lpcbMaxValueLen, lpcbSecurityDescriptor, lpftLastWriteTime);
}

static int RegQueryValueEx (int hKey, TCHAR lpValueName, int lpReserved, int[] lpType, TCHAR lpData, int[] lpcbData) {
	if (IsUnicode) {
		wchar* lpValueName1 = w_str(lpValueName);
		wchar* lpData1 = w_str(lpData);
		return RegQueryValueExW (hKey, lpValueName1, lpReserved, lpType, lpData1, lpcbData);
	}
	char* lpValueName1 = c_str(lpValueName);
	char* lpData1 = c_str(lpData);
	return RegQueryValueExA (hKey, lpValueName1, lpReserved, lpType, lpData1, lpcbData);
}

static int SendMessage (int hWnd, int Msg, int wParam, TCHAR lParam) {
	if (IsUnicode) {
		wchar* lParam1 = w_str(lParam);
		return SendMessageW (hWnd, Msg, wParam, lParam1);
	}
	char* lParam1 = c_str(lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam1);
}

static int SendMessage (int hWnd, int Msg, int [] wParam, int [] lParam) {
	if (IsUnicode) return SendMessageW (hWnd, Msg, wParam, lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam);
}

static int SendMessage (int hWnd, int Msg, int [] wParam, int lParam) {
	if (IsUnicode) return SendMessageW (hWnd, Msg, wParam, lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam);
}

static int SendMessage (int hWnd, int Msg, int wParam, int [] lParam) {
	if (IsUnicode) return SendMessageW (hWnd, Msg, wParam, lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam);
}

static int SendMessage (int hWnd, int Msg, int wParam, short [] lParam) {
	if (IsUnicode) return SendMessageW (hWnd, Msg, wParam, lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam);
}

static int SendMessage (int hWnd, int Msg, int wParam, int lParam) {
	if (IsUnicode) return SendMessageW (hWnd, Msg, wParam, lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam);
}

static int SendMessage (int hWnd, int Msg, int wParam, LVCOLUMN* lParam) {
	if (IsUnicode) return SendMessageW (hWnd, Msg, wParam, lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam);
}

static int SendMessage (int hWnd, int Msg, int wParam, LVHITTESTINFO* lParam) {
	if (IsUnicode) return SendMessageW (hWnd, Msg, wParam, lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam);
}

static int SendMessage (int hWnd, int Msg, int wParam, LVITEM* lParam) {
	if (IsUnicode) return SendMessageW (hWnd, Msg, wParam, lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam);
}

static int SendMessage (int hWnd, int Msg, int wParam, REBARBANDINFO* lParam) {
	if (IsUnicode) return SendMessageW (hWnd, Msg, wParam, lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam);
}

static int SendMessage (int hWnd, int Msg, int wParam, RECT* lParam) {
	if (IsUnicode) return SendMessageW (hWnd, Msg, wParam, lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam);
}

static int SendMessage (int hWnd, int Msg, int wParam, TBBUTTON* lParam) {
	if (IsUnicode) return SendMessageW (hWnd, Msg, wParam, lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam);
}

static int SendMessage (int hWnd, int Msg, int wParam, TBBUTTONINFO* lParam) {
	if (IsUnicode) return SendMessageW (hWnd, Msg, wParam, lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam);
}

static int SendMessage (int hWnd, int Msg, int wParam, TCITEM* lParam) {
	if (IsUnicode) return SendMessageW (hWnd, Msg, wParam, lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam);
}

static int SendMessage (int hWnd, int Msg, int wParam, TOOLINFO* lParam) {
	if (IsUnicode) return SendMessageW (hWnd, Msg, wParam, lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam);
}

static int SendMessage (int hWnd, int Msg, int wParam, TVHITTESTINFO* lParam) {
	if (IsUnicode) return SendMessageW (hWnd, Msg, wParam, lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam);
}

static int SendMessage (int hWnd, int Msg, int wParam, TVINSERTSTRUCT* lParam) {
	if (IsUnicode) return SendMessageW (hWnd, Msg, wParam, lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam);
}

static int SendMessage (int hWnd, int Msg, int wParam, TVITEM* lParam) {
	if (IsUnicode) return SendMessageW (hWnd, Msg, wParam, lParam);
	return SendMessageA (hWnd, Msg, wParam, lParam);
}

static bool SetMenuItemInfo (int hMenu, int uItem, bool fByPosition, MENUITEMINFO* lpmii) {
	if (IsUnicode) return SetMenuItemInfoW (hMenu, uItem, fByPosition, lpmii);
	return SetMenuItemInfoA (hMenu, uItem, fByPosition, lpmii);
}

static int SetWindowLong (int hWnd, int nIndex, int dwNewLong) {
	if (IsUnicode) return SetWindowLongW (hWnd, nIndex, dwNewLong);
	return SetWindowLongA (hWnd, nIndex, dwNewLong);
}

static int SetWindowsHookEx (int idHook, int lpfn, int hMod, int dwThreadId) {
	if (IsUnicode) return SetWindowsHookExW (idHook, lpfn, hMod, dwThreadId);
	return SetWindowsHookExA (idHook, lpfn, hMod, dwThreadId);
}

static bool SetWindowText (int hWnd, TCHAR lpString) {
	if (IsUnicode) {
		wchar* lpString1 = w_str(lpString);
		return SetWindowTextW (hWnd, lpString1);
	}
	char* lpString1 = c_str(lpString);
	return SetWindowTextA (hWnd, lpString1);
}

static int SHBrowseForFolder (BROWSEINFO* lpbi) {
	if (IsUnicode) return  SHBrowseForFolderW(lpbi);
	return SHBrowseForFolderA (lpbi);
}

static bool ShellExecuteEx (SHELLEXECUTEINFO* lpExecInfo) {
	if (IsUnicode) return ShellExecuteExW(lpExecInfo);
	return ShellExecuteExA (lpExecInfo);
}

static bool SHGetPathFromIDList (int pidl, TCHAR pszPath) {
	if (IsUnicode) {
		wchar* pszPath1 = w_str(pszPath);
		return SHGetPathFromIDListW (pidl, pszPath1);
	}
	char* pszPath1 = c_str(pszPath);
	return SHGetPathFromIDListA (pidl, pszPath1);
}

static int StartDoc (int hdc, DOCINFO* lpdi) {
	if (IsUnicode) return StartDocW(hdc, lpdi);
	return StartDocA (hdc, lpdi);
}

static bool SystemParametersInfo (int uiAction, int uiParam, RECT* pvParam, int fWinIni) {
	if (IsUnicode) return SystemParametersInfoW(uiAction, uiParam, pvParam, fWinIni);
	return SystemParametersInfoA (uiAction, uiParam, pvParam, fWinIni);
}

static bool SystemParametersInfo (int uiAction, int uiParam, NONCLIENTMETRICS* pvParam, int fWinIni) {
	if (IsUnicode) return SystemParametersInfoW (uiAction, uiParam, cast(NONCLIENTMETRICSW*)pvParam, fWinIni);
	return SystemParametersInfoA (uiAction, uiParam, cast(NONCLIENTMETRICSA*)pvParam, fWinIni);
}

static bool SystemParametersInfo (int uiAction, int uiParam, int[] pvParam, int fWinIni) {
	if (IsUnicode) return SystemParametersInfoW (uiAction, uiParam, pvParam, fWinIni);
	return SystemParametersInfoA (uiAction, uiParam, pvParam, fWinIni);
}

static int TranslateAccelerator (int hWnd, int hAccTable, MSG lpMsg) {
	if (IsUnicode) return TranslateAcceleratorW (hWnd, hAccTable, lpMsg);
	return TranslateAcceleratorA (hWnd, hAccTable, lpMsg);
}

static bool UnregisterClass (TCHAR lpClassName, int hInstance) {
	if (IsUnicode) {
		wchar* lpClassName1 = w_str(lpClassName);
		return UnregisterClassW(lpClassName1, hInstance);
	}
	char* lpClassName1 = c_str(lpClassName);
	return UnregisterClassA (lpClassName1, hInstance);
}

static short VkKeyScan (short ch) {
	if (IsUnicode) return VkKeyScanW (ch);
	return VkKeyScanA (ch);
}





/** Natives */

static extern (Windows)
{
    int AbortDoc (int hdc);
    int ActivateKeyboardLayout(int hkl, int Flags);
    bool AdjustWindowRectEx (RECT* lpRect, int dwStyle, bool bMenu, int dwExStyle);
    bool Arc (int hdc,int nLeftRect,int nTopRect,int nRightRect,int nBottomRect,int nXStartArc,int nYStartArc,int nXEndArc,int nYEndArc);
    int BeginDeferWindowPos (int nNumWindows);
    int BeginPaint (int hWnd, PAINTSTRUCT* lpPaint);
    bool BitBlt (int hdcDest,int nXDest,int nYDest,int nWidth,int nHeight,int hdcSrc,int nXSrc,int nYSrc,int dwRop);
    bool BringWindowToTop (int hWnd);
    int Call (int address, DLLVERSIONINFO* arg0);
    int CallNextHookEx(int hhk, int nCode, int wParam, int lParam);
    int CallWindowProcW (int lpPrevWndFunc, int hWnd, int Msg, int wParam, int lParam);
    int CallWindowProcA (int lpPrevWndFunc, int hWnd, int Msg, int wParam, int lParam);
    short CharLowerW (short ch);
    short CharLowerA (short ch);
    short CharUpperW (short ch);
    short CharUpperA (short ch);
    bool CheckMenuItem (int hmenu, int uIDCheckItem, int uCheck);
    //bool ChooseColorW (CHOOSECOLOR* lpcc);
    //bool ChooseColorA (CHOOSECOLOR* lpcc);
    //bool ChooseFontW (CHOOSEFONT chooseFont);
    //bool ChooseFontA (CHOOSEFONT chooseFont);
    bool ClientToScreen (int hWnd, POINT* lpPoint);
    bool CloseClipboard ();
    int CombineRgn (int hrgnDest, int hrgnSrc1, int hrgnSrc2, int fnCombineMode);
    bool CommandBar_AddAdornments (int hwndCB, int dwFlags, int dwReserved);
    int CommandBar_Create (int hInst, int hwndParent, int idCmdBar);
    void CommandBar_Destroy (int hwndCB);
    bool CommandBar_DrawMenuBar (int hwndCB, int iButton);
    int CommandBar_Height (int hdnwCB);
    bool CommandBar_InsertMenubarEx (int hwndCB, int hInst, int pszMenu, int iButton);
    bool CommandBar_Show (int hwndCB, bool fShow);
    int CommDlgExtendedError ();
    int CopyImage (int hImage, int uType, int cxDesired, int cyDesired, int fuFlags);
    int CreateAcceleratorTableW (wchar* lpaccl, int cEntries);
    int CreateAcceleratorTableA (char* lpaccl, int cEntries);
    int CreateBitmap (int nWidth, int nHeight, int cPlanes, int cBitsPerPel, byte* lpvBits);
    bool CreateCaret (int hWnd, int hBitmap, int nWidth, int nHeight);
    int CreateCompatibleBitmap (int hdc, int nWidth, int nHeight);

    int CreateCompatibleDC (int hdc);
    int CreateCursor (int hInst, int xHotSpot, int yHotSpot, int nWidth, int nHeight, byte[]* pvANDPlane, byte[]* pvXORPlane);
    int CreateDCW (wchar* lpszDriver, wchar* lpszDevice, int lpszOutput, int lpInitData);
    int CreateDCA (char* lpszDriver, char* lpszDevice, int lpszOutput, int lpInitData);
    int CreateDIBSection(int hdc, BITMAPINFOHEADER* pbmi, int iUsage, byte** ppvBits, int hSection, int dwOffset);
    int CreateFontIndirectW (LOGFONTW* lplf);
    int CreateFontIndirectA (LOGFONTA* lplf);
    int CreateIconIndirect (ICONINFO* lplf);
    int CreateMenu ();
    int CreatePalette (byte[] logPalette);
    int CreatePatternBrush (int colorRef);
    int CreatePen (int fnPenStyle, int nWidth, int crColor);
    int CreatePolygonRgn(int[] lppt, int cPoints, int fnPolyFillMode);
    int CreatePopupMenu ();
    int CreateRectRgn (int left, int top, int right, int bottom);
    int CreateSolidBrush (int colorRef);
    int CreateStreamOnHGlobal(int hGlobal, bool fDeleteOnRelease, int[] ppstm);
    int CreateWindowExW (int dwExStyle, wchar* lpClassName, wchar* lpWindowName, int dwStyle, int X, int Y, int nWidth, int nHeight, int hWndParent, int hMenu, int hInstance, CREATESTRUCT* lpParam);
    int CreateWindowExA (int dwExStyle, char* lpClassName, char* lpWindowName, int dwStyle, int X, int Y, int nWidth, int nHeight, int hWndParent, int hMenu, int hInstance, CREATESTRUCT* lpParam);
    int DeferWindowPos (int hWinPosInfo, int hWnd, int hWndInsertAfter, int X, int Y, int cx, int cy, int uFlags);
    int DefMDIChildProcW (int hWnd, int Msg, int wParam, int lParam);
    int DefMDIChildProcA (int hWnd, int Msg, int wParam, int lParam);
    int DefFrameProcW (int hWnd, int hWndMDIClient, int Msg, int wParam, int lParam);
    int DefFrameProcA (int hWnd, int hWndMDIClient, int Msg, int wParam, int lParam);
    int DefWindowProcW (int hWnd, int Msg, int wParam, int lParam);
    int DefWindowProcA (int hWnd, int Msg, int wParam, int lParam);
    bool DeleteDC (int hdc);
    bool DeleteMenu (int hMenu, int uPosition, int uFlags);
    bool DeleteObject (int hGdiObj);
    bool DestroyAcceleratorTable (int hAccel);
    bool DestroyCaret ();
    bool DestroyCursor (int hCursor);
    bool DestroyIcon (int hIcon);
    bool DestroyMenu (int hMenu);
    bool DestroyWindow (int hWnd);
    int DispatchMessageW (MSG* lpmsg);
    int DispatchMessageA (MSG* lpmsg);
    bool DragDetect (int hwnd, POINT pt);
    void DragFinish (int hDrop);
    int DragQueryFileA (int hDrop, int iFile, char* lpszFile, int cch);
    int DragQueryFileW (int hDrop, int iFile, wchar* lpszFile, int cch);
    bool DrawEdge (int hdc, RECT* qrc, int edge, int grfFlags);
    bool DrawFocusRect (int hDC, RECT* lpRect);
    bool DrawFrameControl (int hdc, RECT* lprc, int uType, int uState);
    bool DrawIconEx (int hdc, int xLeft, int yTop, int hIcon, int cxWidth, int cyWidth, int istepIfAniCur, int hbrFlickerFreeDraw, int diFlags);
    bool DrawMenuBar (int hWnd);
    bool DrawStateW (int hdc, int hbr, int lpOutputFunc, int lData, int wData, int x, int y, int cx, int cy, int fuFlags);
    bool DrawStateA (int hdc, int hbr, int lpOutputFunc, int lData, int wData, int x, int y, int cx, int cy, int fuFlags);
    int DrawTextW (int hDC, wchar* lpString, int nCount, RECT* lpRect, int uFormat);
    int DrawTextA (int hDC, char* lpString, int nCount, RECT* lpRect, int uFormat);
    bool Ellipse (int hdc,int nLeftRect,int nTopRect,int nRightRect,int nBottomRect);
    bool EnableMenuItem (int hMenu, int uIDEnableItem, int uEnable);
    bool EnableScrollBar (int hWnd, int wSBflags, int wArrows);
    bool EnableWindow (int hWnd, bool bEnable);
    bool EnumSystemLanguageGroupsW(int pLangGroupEnumProc, int dwFlags, int lParam);
    bool EnumSystemLanguageGroupsA(int pLangGroupEnumProc, int dwFlags, int lParam);
    bool EnumSystemLocalesW(void* lpLocaleEnumProc, int dwFlags);
    bool EnumSystemLocalesA(void* lpLocaleEnumProc, int dwFlags);
    bool EndDeferWindowPos (int hWinPosInfo);
    int EndDoc (int hdc);
    int EndPage (int hdc);
    int EndPaint (int hWnd, PAINTSTRUCT* lpPaint);
    bool EnumDisplayMonitors (int hdc, RECT* lprcClip, int lpfnEnum, int dwData);
    int EnumFontFamiliesW (int hdc, wchar* lpszFamily, int lpEnumFontFamProc, int lParam);
    int EnumFontFamiliesA (int hdc, char* lpszFamily, int lpEnumFontFamProc, int lParam);
    bool EqualRect (RECT* lprc1, RECT* lprc2);
    bool EqualRgn (int hSrcRgn1, int hSrcRgn2);
    int ExpandEnvironmentStringsW (wchar* lpSrc, wchar* lsDst, int nSize);
    int ExpandEnvironmentStringsA (char* lpSrc, char* lsDst, int nSize);
    bool ExtTextOutW(int hdc, int X, int Y, int fuOptions, RECT* lprc, wchar* lpString, int cbCount, int[] lpDx);
    bool ExtTextOutA(int hdc, int X, int Y, int fuOptions, RECT* lprc, char* lpString, int cbCount, int[] lpDx);
    int ExtractIconExW (wchar* lpszFile, int nIconIndex, int [] phiconLarge, int [] phiconSmall, int nIcons);
    int ExtractIconExA (char* lpszFile, int nIconIndex, int [] phiconLarge, int [] phiconSmall, int nIcons);
    int FillRect(int hDC, RECT* lprc, int hbr);
    int FindWindowA (char* lpClassName, char* lpWindowName);
    int FindWindowW (wchar* lpClassName, wchar* lpWindowName);
    bool FreeLibrary (int hLibModule);
    int GetACP ();
    int GetActiveWindow ();
    int GetBkColor (int hDC);
    int GetCapture ();
    bool GetCaretPos (POINT* lpPoint);
    bool GetCharABCWidthsA (int hdc, int iFirstChar, int iLastChar, int [] lpabc);
    bool GetCharABCWidthsW (int hdc, int iFirstChar, int iLastChar, int [] lpabc);
    int GetCharacterPlacementW(int hdc, wchar* lpString, int nCount, int nMaxExtent, GCP_RESULTS* lpResults, int dwFlags);
    int GetCharacterPlacementA(int hdc, char* lpString, int nCount, int nMaxExtent, GCP_RESULTS* lpResults, int dwFlags);
    bool GetCharWidthA (int hdc, int iFirstChar, int iLastChar, int [] lpBuffer);
    bool GetCharWidthW (int hdc, int iFirstChar, int iLastChar, int [] lpBuffer);
    bool GetClassInfoW (int hInstance, wchar* lpClassName, WNDCLASS* lpWndClass);
    bool GetClassInfoA (int hInstance, char* lpClassName, WNDCLASS* lpWndClass);
    bool GetClientRect (int hWnd, RECT* lpRect);
    int GetClipboardData (int uFormat);
    int GetClipboardFormatNameA (int format, char* lpszFormatName, int cchMaxCount);
    int GetClipboardFormatNameW (int format, wchar* lpszFormatName, int cchMaxCount);
    int GetClipBox (int hdc, RECT* lprc);
    int GetClipRgn (int hdc, int hrgn);
    //bool GetComboBoxInfo (int hwndCombo, COMBOBOXINFO* pcbi);
    int GetCurrentObject (int hdc, int uObjectType);
    int GetCurrentProcessId ();
    int GetCurrentThreadId ();
    int GetCursor ();
    bool GetCursorPos (POINT* lpPoint);
    int GetDC (int hwnd);
    int GetDCEx (int hWnd, int hrgnClip, int flags);
    int GetDesktopWindow ();
    int GetDeviceCaps (int hdc, int nIndex);
    int GetDialogBaseUnits ();
    int GetDIBColorTable (int hdc, int uStartIndex, int cEntries, byte* pColors);
    int GetDIBits (int hdc, int hbmp, int uStartScan, int cScanLines, byte* lpvBits, BITMAPINFOHEADER* lpbi, int uUsage);
    int GetDlgItem (int hDlg, int nIDDlgItem);
    int GetDoubleClickTime ();
    int GetFocus ();
    int GetFontLanguageInfo(int hdc);
    bool GetIconInfo (int hIcon, ICONINFO* piconinfo);
    int GetKeyboardLayoutList(int nBuff, int[] lpList);
    int GetKeyboardLayout(int idThread);short GetKeyState (int nVirtKey);
    bool GetKeyboardState (char* lpKeyState);
    int GetKeyNameTextW(int lParam, wchar* lpString, int nSize);
    int GetKeyNameTextA(int lParam, char* lpString, int nSize);
    int GetLastActivePopup (int hWnd);
    int GetLastError ();
    int GetLayout (int hdc);
    /* returns the instance handle to the swt library */
    int GetLibraryHandle ();
    int GetLocaleInfoW (int Locale, int LCType, wchar* lpLCData, int cchData);
    int GetLocaleInfoA (int Locale, int LCType, char* lpLCData, int cchData);
    int GetMenu (int hWnd);
    int GetMenuDefaultItem (int hMenu, int fByPos, int gmdiFlags);
    bool GetMenuInfo (int hmenu, MENUINFO* lpcmi);
    int GetMenuItemCount (int hMenu);
    bool GetMenuItemInfoW (int hMenu, int uItem, bool fByPosition, MENUITEMINFO* lpmii);
    bool GetMenuItemInfoA (int hMenu, int uItem, bool fByPosition, MENUITEMINFO* lpmii);
    bool GetMessageW (MSG lpMsg, int hWnd, int wMsgFilterMin, int wMsgFilterMax);
    bool GetMessageA (MSG lpMsg, int hWnd, int wMsgFilterMin, int wMsgFilterMax);
    int GetMessagePos ();
    int GetMessageTime ();
    int GetTextCharset(int hdc);
    int GetTickCount ();
    int GetModuleHandleW (wchar* lpModuleName);
    int GetModuleHandleA (char* lpModuleName);
    bool GetMonitorInfoW (int hmonitor, MONITORINFO* lpmi);
    bool GetMonitorInfoA (int hmonitor, MONITORINFO* lpmi);
    int GetNearestPaletteIndex(int hPal, int crColor);

    int GetObjectA (int hgdiobj, int cbBuffer, void* lpvObject);
    int GetObjectW (int hgdiobj, int cbBuffer, void* lpvObject);
    int GetObjectA (int hgdiobj, int cbBuffer, BITMAP* lpvObject);
    int GetObjectW (int hgdiobj, int cbBuffer, BITMAP* lpvObject);
    int GetObjectA (int hgdiobj, int cbBuffer, DIBSECTION* lpvObject);
    int GetObjectW (int hgdiobj, int cbBuffer, DIBSECTION* lpvObject);
    int GetObjectA (int hgdiobj, int cbBuffer, LOGBRUSH* lpvObject);
    int GetObjectW (int hgdiobj, int cbBuffer, LOGBRUSH* lpvObject);
    int GetObjectA (int hgdiobj, int cbBuffer, LOGFONTA* lpvObject);
    int GetObjectW (int hgdiobj, int cbBuffer, LOGFONTW* lpvObject);
    int GetObjectA (int hgdiobj, int cbBuffer, LOGPEN* lpvObject);
    int GetObjectW (int hgdiobj, int cbBuffer, LOGPEN* lpvObject);
    bool GetOpenFileNameW (OPENFILENAME* lpofn);
    bool GetOpenFileNameA (OPENFILENAME* lpofn);
    int GetPaletteEntries (int hPalette, int iStartIndex, int nEntries, byte[] logPalette);
    int GetParent (int hWnd);
    int GetPixel (int hdc, int x, int y);
    int GetProcAddress (int hModule, char[]* lpProcName);
    int GetProcessHeap ();
    int GetProfileStringW (wchar* lpAppName, wchar* lpKeyName, wchar* lpDefault, wchar* lpReturnedString, int nSize);
    int GetProfileStringA (char* lpAppName, char* lpKeyName, char* lpDefault, char* lpReturnedString, int nSize);
    int GetRegionData (int hRgn, int dwCount, int [] lpRgnData);
    int GetRgnBox (int hrgn, RECT* lprc);
    int GetROP2 (int hdc);
    bool GetSaveFileNameW (OPENFILENAME* lpofn);
    bool GetSaveFileNameA (OPENFILENAME* lpofn);
    bool GetScrollInfo (int hwnd, int flags, SCROLLINFO* info);
    int GetStockObject (int fnObject);
    int GetSysColor (int nIndex);
    int GetSysColorBrush (int nIndex);
    int GetSystemMenu (int hWnd, bool bRevert);
    int GetSystemMetrics (int nIndex);
    int GetSystemPaletteEntries(int hdc, int iStartIndex, int nEntries, byte[] lppe);
    int GetTextColor (int hDC);
    bool GetTextExtentPoint32W (int hdc, wchar* lpString, int cbString, SIZE* lpSize);
    bool GetTextExtentPoint32A (int hdc, char* lpString, int cbString, SIZE* lpSize);
    bool GetTextMetricsW (int hdc, TEXTMETRICW* lptm);
    bool GetTextMetricsA (int hdc, TEXTMETRICA* lptm);
    bool GetUpdateRect (int hWnd, RECT* lpRect, bool bErase);
    int GetUpdateRgn (int hWnd, int hRgn, bool bErase);
    bool GetVersionExW (OSVERSIONINFOW* lpVersionInfo);
    bool GetVersionExA (OSVERSIONINFOA* lpVersionInfo);
    int GetWindow (int hWnd, int uCmd);
    int GetWindowLongW (int hWnd, int nIndex);
    int GetWindowLongA (int hWnd, int nIndex);
    bool GetWindowPlacement (int hWnd, WINDOWPLACEMENT* lpwndpl);
    bool GetWindowRect (int hWnd, RECT* lpRect);
    int GetWindowRgn(int hWnd, int hRgn);
    int GetWindowTextW (int hWnd, wchar* lpString, int nMaxCount);
    int GetWindowTextA (int hWnd, char* lpString, int nMaxCount);
    int GetWindowTextLengthW (int hWnd);
    int GetWindowTextLengthA (int hWnd);
    int GetWindowThreadProcessId (int hWnd, int [] lpdwProcessId);
    int GlobalAlloc (int uFlags, int dwBytes);
    int GlobalFree (int hMem);
    int GlobalLock (int hMem);
    int GlobalSize (int hMem);
    bool GlobalUnlock (int hMem);
    bool GradientFill(int hdc, TRIVERTEX* pVertex, int dwNumVertex, void* pMesh, int dwNumMesh, int dwMode);
    int HeapAlloc (int hHeap, int dwFlags, int dwBytes);
    bool HeapFree (int hHeap, int dwFlags, int lpMem);
    bool HideCaret (int hWnd);
    int ImageList_Add (int himl, int hbmImage, int hbmMask);
    int ImageList_AddMasked (int himl, int hbmImage, int crMask);
    int ImageList_Create (int cx, int cy, int flags, int cInitial, int cGrow);
    bool ImageList_Destroy (int himl);
    int ImageList_GetIcon (int himl, int i, int flags);
    bool ImageList_GetIconSize (int himl, int [] cx, int [] cy);
    int ImageList_GetImageCount (int himl);
    bool ImageList_Remove (int himl, int i);
    bool ImageList_Replace (int himl, int i, int hbmImage, int hbmMask);
    int ImageList_ReplaceIcon (int himl, int i, int hicon);
    bool ImageList_SetIconSize (int himl, int cx, int cy);
    int ImmAssociateContext (int hWnd, int hIMC);
    int ImmCreateContext ();
    bool ImmDestroyContext (int hIMC);
    bool ImmGetCompositionFontW (int hIMC, LOGFONTW* lplf);
    bool ImmGetCompositionFontA (int hIMC, LOGFONTA* lplf);
    int ImmGetCompositionStringW (int hIMC, int dwIndex, wchar* lpBuf, int dwBufLen);
    int ImmGetCompositionStringA (int hIMC, int dwIndex, char* lpBuf, int dwBufLen);
    int ImmGetContext (int hWnd);
    bool ImmGetConversionStatus (int hIMC, int [] lpfdwConversion, int [] lpfdwSentence);
    int ImmGetDefaultIMEWnd (int hWnd);
    bool ImmGetOpenStatus (int hIMC);
    bool ImmReleaseContext (int hWnd, int hIMC);
    bool ImmSetCompositionFontW (int hIMC, LOGFONTW* lplf);
    bool ImmSetCompositionFontA (int hIMC, LOGFONTA* lplf);
    bool ImmSetCompositionWindow (int hIMC, COMPOSITIONFORM lpCompForm);
    bool ImmSetConversionStatus (int hIMC, int fdwConversion, int dwSentence);
    bool ImmSetOpenStatus (int hIMC, bool fOpen);
    void InitCommonControls ();
    bool InitCommonControlsEx (INITCOMMONCONTROLSEX lpInitCtrls);
    bool InsertMenuW (int hMenu, int uPosition, int uFlags, int uIDNewItem, wchar* lpNewItem);
    bool InsertMenuA (int hMenu, int uPosition, int uFlags, int uIDNewItem, char* lpNewItem);
    bool InsertMenuItemW (int hMenu, int uItem, bool fByPosition, MENUITEMINFO* lpmii);
    bool InsertMenuItemA (int hMenu, int uItem, bool fByPosition, MENUITEMINFO* lpmii);
    bool InvalidateRect(int hWnd, RECT* lpRect, bool bErase);
    bool InvalidateRgn (int hWnd, int hRgn, bool bErase);
    bool IsDBCSLeadByte (byte TestChar);
    bool IsIconic (int hWnd);
    //static int IsPPC();
    //static int IsSP();
    bool IsWindowEnabled (int hWnd);
    bool IsWindowVisible (int hWnd);
    bool IsZoomed (int hWnd);
    bool KillTimer (int hWnd, int uIDEvent);
    bool LineTo (int hdc,int x1, int x2);
    int LoadBitmapW (int hInstance, int lpBitmapName);
    int LoadBitmapA (int hInstance, int lpBitmapName);
    int LoadCursorW (int hInstance, int lpCursorName);
    int LoadCursorA (int hInstance, int lpCursorName);
    int LoadIconW (int hInstance, int lpIconName);
    int LoadIconA (int hInstance, int lpIconName);
    int LoadImageW (int hinst, wchar* lpszName, int uType, int cxDesired, int cyDesired, int fuLoad);
    int LoadImageA (int hinst, char* lpszName, int uType, int cxDesired, int cyDesired, int fuLoad);
    int LoadLibraryW (wchar* lpLibFileName);
    int LoadLibraryA (char* lpLibFileName);
    int MapVirtualKeyW (int uCode, int uMapType);
    int MapVirtualKeyA (int uCode, int uMapType);
    int MapWindowPoints (int hWndFrom, int hWndTo, POINT* lpPoints, int cPoints);
    int MapWindowPoints (int hWndFrom, int hWndTo, RECT* lpPoints, int cPoints);
    bool MessageBeep (int uType);
    int MessageBoxW (int hWnd, wchar* lpText, wchar* lpCaption, int uType);
    int MessageBoxA (int hWnd, char* lpText, char* lpCaption, int uType);
    int MonitorFromWindow (int hwnd, int dwFlags);

	/+ 2004.5.23 BSA
	 +	rem'd out the general MoveMemory and uncommented the others
	 +	(Andy's work, maybe?)
	 +/

    // :P
    //void MoveMemory(void* destination, void* source, int length);

    void MoveMemory (char[] Destination, int SourcePtr, int Length);
    void MoveMemory (char* Destination, int Source, int Length);
    void MoveMemory (char* Destination, ACCEL Source, int Length);
    void MoveMemory (char* Destination, BITMAPINFOHEADER Source, int Length);
    void MoveMemory (int [] Destination, int Source, int Length);
    void MoveMemory (int Destination, byte[] Source, int Length);
//    void MoveMemory (int Destination, char* Source, int Length);
    void MoveMemory (int Destination, wchar* Source, int Length);
    void MoveMemory (int Destination, int [] Source, int Length);
    void MoveMemory (int Destination, GRADIENT_RECT Source, int Length);
    void MoveMemory (int Destination, LOGFONTW Source, int Length);
    void MoveMemory (int Destination, LOGFONTA Source, int Length);
    void MoveMemory (int Destination, MEASUREITEMSTRUCT Source, int Length);
    void MoveMemory (int Destination, MSG Source, int Length);
    void MoveMemory (int Destination, NMTTDISPINFOW Source, int Length);
    void MoveMemory (int Destination, NMTTDISPINFOA Source, int Length);
    void MoveMemory (int Destination, RECT* Source, int Length);
    void MoveMemory (int Destination, TRIVERTEX Source, int Length);
    void MoveMemory (int Destination, WINDOWPOS* Source, int Length);
    void MoveMemory (BITMAPINFOHEADER* Destination, char* Source, int Length);
    void MoveMemory (DRAWITEMSTRUCT Destination, int Source, int Length);
    //void MoveMemory (HDITEM Destination, int Source, int Length);
    void MoveMemory (HELPINFO* Destination, int Source, int Length);
    void MoveMemory (LOGFONTW Destination, int Source, int Length);
    void MoveMemory (LOGFONTA Destination, int Source, int Length);
    void MoveMemory (MEASUREITEMSTRUCT Destination, int Source, int Length);
    void MoveMemory (NMHDR* Destination, int Source, int Length);
//    void MoveMemory (NMRGINFO* Destination, int Source, int Length);
//    void MoveMemory (NMCUSTOMDRAW* Destination, int Source, int Length);
//    void MoveMemory (NMLVCUSTOMDRAW* Destination, int Source, int Length);
//    void MoveMemory (NMTVCUSTOMDRAW* Destination, int Source, int Length);
//    void MoveMemory (int Destination, NMLVCUSTOMDRAW* Source, int Length);
//    void MoveMemory (int Destination, NMTVCUSTOMDRAW* Source, int Length);
//    void MoveMemory (int Destination, NMLVDISPINFO* Source, int Length);
//    void MoveMemory (NMLVDISPINFO* Destination, int Source, int Length);
//    void MoveMemory (NMHEADER* Destination, int Source, int Length);
//    void MoveMemory (NMLISTVIEW* Destination, int Source, int Length);
//    void MoveMemory (NMREBARCHEVRON Destination, int Source, int Length);
    //void MoveMemory (NMTOOLBAR Destination, int Source, int Length);
    void MoveMemory (NMTTDISPINFOW Destination, int Source, int Length);
    void MoveMemory (NMTTDISPINFOA Destination, int Source, int Length);
    void MoveMemory (TVITEM Destination, int Source, int Length);
    void MoveMemory (WINDOWPOS* Destination, int Source, int Length);
    void MoveMemory (MSG Destination, int Source, int Length);
    //void MoveMemory(int Destination, DROPFILES Source, int Length);
    void MoveMemory(double[] Destination, int SourcePtr, int Length);
    void MoveMemory(float[] Destination, int SourcePtr, int Length);
    void MoveMemory(short[] Destination, int SourcePtr, int Length);
    void MoveMemory(int DestinationPtr, double[] Source, int Length);
    void MoveMemory(int DestinationPtr, float[] Source, int Length);
    void MoveMemory(int DestinationPtr, short[] Source, int Length);


    bool MoveToEx (int hdc,int x1, int x2, int lPoint);
    int MsgWaitForMultipleObjectsEx (int nCount, int pHandles, int dwMilliseconds, int dwWakeMask, int dwFlags);
    int MultiByteToWideChar (int CodePage, int dwFlags, char* lpMultiByteStr, int cchMultiByte, wchar* lpWideCharStr, int cchWideChar);
    int MultiByteToWideChar (int CodePage, int dwFlags, int lpMultiByteStr, int cchMultiByte, wchar* lpWideCharStr, int cchWideChar);
    int OleInitialize (int pvReserved);
    void OleUninitialize ();
    bool OpenClipboard (int hWndNewOwner);
    bool PatBlt (int hdc,int x1, int x2,int w, int h, int rop);
    bool PeekMessageW (MSG* lpMsg, int hWnd, int wMsgFilterMin, int wMsgFilterMax, int wRemoveMsg);
    bool PeekMessageA (MSG* lpMsg, int hWnd, int wMsgFilterMin, int wMsgFilterMax, int wRemoveMsg);
    bool Pie (int hdc, int nLeftRect, int nTopRect, int nRightRect, int nBottomRect, int nXStartArc, int nYStartArc, int nXEndArc, int nYEndArc);
    bool Polygon (int hdc, int [] points, int nPoints);
    bool Polyline (int hdc,int[] points, int nPoints);
    bool PostMessageW (int hWnd, int Msg, int wParam, int lParam);
    bool PostMessageA (int hWnd, int Msg, int wParam, int lParam);
    bool PostThreadMessageW (int idThread, int Msg, int wParam, int lParam);
    bool PostThreadMessageA (int idThread, int Msg, int wParam, int lParam);
    bool PrintDlgW (PRINTDLG lppd);
    bool PrintDlgA (PRINTDLG lppd);
    bool PtInRect (RECT* rect, POINT* pt);
    bool PtInRegion (int hrgn, int X, int Y);
    int RealizePalette(int hDC);
    bool Rectangle (int hdc, int nLeftRect, int nTopRect, int nRightRect, int nBottomRect);
    bool RectInRegion (int hrgn, RECT* lprc);
    bool RedrawWindow (int hWnd, RECT* lprcUpdate, int hrgnUpdate, int flags);
    int RegCloseKey (int hKey);
    int RegisterClassW (WNDCLASS* lpWndClass);
    int RegisterClassA (WNDCLASS* lpWndClass);
    int RegEnumKeyExW (int hKey, int dwIndex, wchar* lpName, int [] lpcName, int [] lpReserved, wchar* lpClass, int [] lpcClass, FILETIME* lpftLastWriteTime);
    int RegisterClipboardFormatA (char* lpszFormat);
    int RegisterClipboardFormatW (wchar* lpszFormat);
    int RegOpenKeyExW (int hKey, wchar* lpSubKey, int ulOptions, int samDesired, int[] phkResult);
    int RegQueryInfoKeyW (int hKey, int lpClass, int[] lpcbClass, int lpReserved, int[] lpSubKeys, int[] lpcbMaxSubKeyLen, int[] lpcbMaxClassLen, int[] lpcValues, int[] lpcbMaxValueNameLen, int[] lpcbMaxValueLen, int[] lpcbSecurityDescriptor, int lpftLastWriteTime);
    int RegQueryValueExW (int hKey, wchar* lpValueName, int lpReserved, int[] lpType, wchar* lpData, int[] lpcbData);
    int RegEnumKeyExA (int hKey, int dwIndex, char* lpName, int [] lpcName, int [] lpReserved, char* lpClass, int [] lpcClass, FILETIME* lpftLastWriteTime);
    int RegOpenKeyExA (int hKey, char* lpSubKey, int ulOptions, int samDesired, int[] phkResult);
    int RegQueryInfoKeyA (int hKey, int lpClass, int[] lpcbClass, int lpReserved, int[] lpSubKeys, int[] lpcbMaxSubKeyLen, int[] lpcbMaxClassLen, int[] lpcValues, int[] lpcbMaxValueNameLen, int[] lpcbMaxValueLen, int[] lpcbSecurityDescriptor, int lpftLastWriteTime);
    int RegQueryValueExA (int hKey, char* lpValueName, int lpReserved, int[] lpType, char* lpData, int[] lpcbData);
    bool ReleaseCapture ();
    int ReleaseDC (int hWnd, int hDC);
    bool RemoveMenu (int hMenu, int uPosition, int uFlags);
    bool RoundRect (int hdc, int nLeftRect, int nTopRect, int nRightRect, int nBottomRect, int nWidth, int nHeight);
    bool ScreenToClient (int hWnd, POINT* lpPoint);
    int ScrollWindowEx (int hWnd, int dx, int dy, RECT* prcScroll, RECT* prcClip, int hrgnUpdate, RECT* prcUpdate, int flags);
    int SelectClipRgn (int hdc, int hrgn);
    int SelectObject(int hDC, int HGDIObj);
    int SelectPalette(int hDC, int hpal, bool bForceBackground);

//    int SendMessageW (int hWnd, int msg, int wparam, int lparam);
//    int SendMessageA (int hWnd, int msg, int wparam, int lparam);

    // D doesn't allow extern(Windows) functions to be overloaded in this manner
//    /*
    int SendMessageW (int hWnd, int Msg, int [] wParam, int [] lParam);
    int SendMessageW (int hWnd, int Msg, int [] wParam, int lParam);
    int SendMessageW (int hWnd, int Msg, int wParam, wchar* lParam);
    int SendMessageW (int hWnd, int Msg, int wParam, int [] lParam);
    int SendMessageW (int hWnd, int Msg, int wParam, short [] lParam);
    int SendMessageW (int hWnd, int Msg, int wParam, int lParam);
    int SendMessageW (int hWnd, int Msg, int wParam, LVCOLUMN* lParam);
    int SendMessageW (int hWnd, int Msg, int wParam, LVHITTESTINFO* lParam);
    int SendMessageW (int hWnd, int Msg, int wParam, LVITEM* lParam);
    int SendMessageW (int hWnd, int Msg, int wParam, REBARBANDINFO* lParam);
    int SendMessageW (int hWnd, int Msg, int wParam, RECT* lParam);
    int SendMessageW (int hWnd, int Msg, int wParam, TBBUTTON* lParam);
    int SendMessageW (int hWnd, int Msg, int wParam, TBBUTTONINFO* lParam);
    int SendMessageW (int hWnd, int Msg, int wParam, TCITEM* lParam);
    int SendMessageW (int hWnd, int Msg, int wParam, TOOLINFO* lParam);
    int SendMessageW (int hWnd, int Msg, int wParam, TVHITTESTINFO* lParam);
    int SendMessageW (int hWnd, int Msg, int wParam, TVINSERTSTRUCT* lParam);
    int SendMessageW (int hWnd, int Msg, int wParam, TVITEM* lParam);
    int SendMessageA (int hWnd, int Msg, int [] wParam, int [] lParam);
    int SendMessageA (int hWnd, int Msg, int [] wParam, int lParam);
    int SendMessageA (int hWnd, int Msg, int wParam, char* lParam);
    int SendMessageA (int hWnd, int Msg, int wParam, int [] lParam);
    int SendMessageA (int hWnd, int Msg, int wParam, short [] lParam);
    int SendMessageA (int hWnd, int Msg, int wParam, int lParam);
    int SendMessageA (int hWnd, int Msg, int wParam, LVCOLUMN* lParam);
    int SendMessageA (int hWnd, int Msg, int wParam, LVHITTESTINFO* lParam);
    int SendMessageA (int hWnd, int Msg, int wParam, LVITEM* lParam);
    int SendMessageA (int hWnd, int Msg, int wParam, REBARBANDINFO* lParam);
    int SendMessageA (int hWnd, int Msg, int wParam, RECT* lParam);
    int SendMessageA (int hWnd, int Msg, int wParam, TBBUTTON* lParam);
    int SendMessageA (int hWnd, int Msg, int wParam, TBBUTTONINFO* lParam);
    int SendMessageA (int hWnd, int Msg, int wParam, TCITEM* lParam);
    int SendMessageA (int hWnd, int Msg, int wParam, TOOLINFO* lParam);
    int SendMessageA (int hWnd, int Msg, int wParam, TVHITTESTINFO* lParam);
    int SendMessageA (int hWnd, int Msg, int wParam, TVINSERTSTRUCT* lParam);
    int SendMessageA (int hWnd, int Msg, int wParam, TVITEM* lParam);
//    */

    int SetActiveWindow (int hWnd);
    int SetBkColor (int hdc, int colorRef);
    int SetBkMode (int hdc, int mode);
    int SetCapture (int hWnd);
    bool SetCaretPos (int X, int Y);
    int SetClipboardData (int uFormat, int hMem);
    int SetCursor (int hCursor);
    bool SetCursorPos (int X, int Y);
    int SetDIBColorTable (int hdc, int uStartIndex, int cEntries, byte[] pColors);
    int SetErrorMode (int uMode);
    int SetFocus (int hWnd);
    bool SetForegroundWindow (int hWnd);
    int SetLayout (int hdc, int dwLayout);
    bool SetMenu (int hWnd, int hMenu);
    bool SetMenuDefaultItem (int hMenu, int uItem, int fByPos);
    bool SetMenuInfo (int hmenu, MENUINFO* lpcmi);
    bool SetMenuItemInfoW (int hMenu, int uItem, bool fByPosition, MENUITEMINFO* lpmii);
    bool SetMenuItemInfoA (int hMenu, int uItem, bool fByPosition, MENUITEMINFO* lpmii);
    int SetPaletteEntries (int hPal, int iStart, int cEntries, byte[] lppe);
    int SetParent (int hWndChild, int hWndNewParent);
    int SetPixel (int hdc, int X, int Y, int crColor);
    bool SetRect(RECT* lprc, int xLeft, int yTop, int xRight, int yBottom);
    bool SetRectRgn (int hrgn, int nLeftRect, int nTopRect, int nRightRect, int nBottomRect);
    int SetROP2 (int hdc, int fnDrawMode);
    bool SetScrollInfo (int hwnd, int flags, SCROLLINFO* info, bool fRedraw);
    int SetStretchBltMode(int hdc, int iStretchMode);
    int SetTextAlign(int hdc, int fMode);
    int SetTextColor (int hdc, int colorRef);
    int SetTimer (int hWnd, int nIDEvent, int Elapse, int lpTimerFunc);
    int SetWindowLongW (int hWnd, int nIndex, int dwNewLong);
    int SetWindowLongA (int hWnd, int nIndex, int dwNewLong);
    bool SetWindowPlacement (int hWnd, WINDOWPLACEMENT* lpwndpl);
    bool SetWindowPos(int hWnd, int hWndInsertAfter, int X, int Y, int cx, int cy, int uFlags);
    int SetWindowRgn(int hWnd, int hRgn, bool bRedraw);
    bool SetWindowTextW (int hWnd, wchar* lpString);
    bool SetWindowTextA (int hWnd, char* lpString);
    int SetWindowsHookExW (int idHook, int lpfn,  int hMod,  int dwThreadId);
    int SetWindowsHookExA (int idHook, int lpfn,  int hMod,  int dwThreadId);
    int SHBrowseForFolderW (BROWSEINFO* lpbi);
    int SHBrowseForFolderA (BROWSEINFO* lpbi);
    bool SHCreateMenuBar(SHMENUBARINFO* pmb);
    bool SHHandleWMSettingChange (int hwnd, int wParam, int lParam, SHACTIVATEINFO* psai);
    int SHRecognizeGesture(SHRGINFO* shrg);
    void SHSendBackToFocusWindow (int uMsg, int wp, int lp);
    bool SHSipPreference (int hwnd, int st);
    bool ShellExecuteExW (SHELLEXECUTEINFO* lpExecInfo);
    bool ShellExecuteExA (SHELLEXECUTEINFO* lpExecInfo);
    int SHGetMalloc (int [] ppMalloc);
    bool SHGetPathFromIDListW (int pidl, wchar* pszPath);
    bool SHGetPathFromIDListA (int pidl, char* pszPath);
    bool SHSetAppKeyWndAssoc(byte bVk, int hwnd);
    bool ShowCaret (int hWnd);
    bool ShowOwnedPopups (int hWnd, bool fShow);
    bool ShowScrollBar (int hWnd, int wBar, bool bShow);
    bool ShowWindow (int hWnd, int nCmdShow);
//    bool SipGetInfo (SIPINFO* pSipInfo);
    int StartDocW (int hdc, DOCINFO* lpdi);
    int StartDocA (int hdc, DOCINFO* lpdi);
    int StartPage (int hdc);
    bool StretchBlt (int hdcDest, int nXOriginDest, int nYOriginDest, int nWidthDest, int nHeightDest, int hdcSrc, int nXOriginSrc, int nYOriginSrc, int nWidthSrc, int nHeightSrc, int dwRop);

    int SystemParametersInfoA(int uiAction, int uiParam, int pvParam, int fWinIni);
    int SystemParametersInfoW(int uiAction, int uiParam, int pvParam, int fWinIni);

    bool SystemParametersInfoW (int uiAction, int uiParam, RECT* pvParam, int fWinIni);
    bool SystemParametersInfoA (int uiAction, int uiParam, RECT* pvParam, int fWinIni);
    bool SystemParametersInfoW (int uiAction, int uiParam, NONCLIENTMETRICSW* pvParam, int fWinIni);
    bool SystemParametersInfoA (int uiAction, int uiParam, NONCLIENTMETRICSA* pvParam, int fWinIni);
    bool SystemParametersInfoW (int uiAction, int uiParam, int[] pvParam, int fWinIni);
    bool SystemParametersInfoA (int uiAction, int uiParam, int[] pvParam, int fWinIni);

    int ToAscii(int uVirtKey, int uScanCode, byte[]* lpKeyState, char[]* lpChar, int uFlags);
    int ToUnicode(int wVirtKey, int wScanCode, byte[]* lpKeyState, wchar* pwszBuff, int cchBuff, int wFlags);
    bool TrackMouseEvent(TRACKMOUSEEVENT* lpEventTrack);
    bool TrackPopupMenu (int hMenu, int uFlags, int x, int y, int nReserved, int hWnd, RECT* prcRect);
    int TranslateAcceleratorW (int hWnd, int hAccTable, MSG lpMsg);
    int TranslateAcceleratorA (int hWnd, int hAccTable, MSG lpMsg);
    bool TranslateCharsetInfo(int lpSrc, int [] lpCs, int dwFlags);
    bool TranslateMDISysAccel (int hWndClient, MSG lpMsg);
    bool TranslateMessage(MSG* lpmsg);
    bool TransparentImage (int hdcDest, int DstX, int DstY, int DstCx, int DstCy,int hSrc, int SrcX, int SrcY, int SrcCx, int SrcCy, int TransparentColor);bool UnhookWindowsHookEx(int hhk);
    bool UnregisterClassW (wchar* lpClassName, int hInstance);
    bool UnregisterClassA (char* lpClassName, int hInstance);
    bool UpdateWindow (int hWnd);
    bool ValidateRect (int hWnd, RECT* lpRect);
    short VkKeyScanW (short ch);
    short VkKeyScanA (short ch);
    int VtblCall (int ppVtbl, int fnNumber, int arg0);
    bool WaitMessage ();
    int WideCharToMultiByte (int CodePage, int dwFlags, wchar* lpWideCharStr, int cchWideChar, char* lpMultiByteStr, int cchMultiByte, char* lpDefaultChar, bool [] lpUsedDefaultChar);
    int WideCharToMultiByte (int CodePage, int dwFlags, wchar* lpWideCharStr, int cchWideChar, int lpMultiByteStr, int cchMultiByte, char* lpDefaultChar, bool [] lpUsedDefaultChar);
    int WindowFromDC (int lpPoint);
    int WindowFromPoint (POINT* lpPoint);
}
}
