/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

/*******************************************************************************
 * DWT: SWT port for the D Language
 * 
 * Contributors:
 * 	The DWT team at http://www.dsource.org/projects/dwt
 * 
 *******************************************************************************/



module dwt.internal.win32.tchar;

/**
 * This class implements the conversions between unicode characters
 * and the <em>platform supported</em> representation for characters.
 * <p>
 * Note that, unicode characters which can not be found in the platform
 * encoding will be converted to an arbitrary platform specific character.
 * </p>
 */

public class TCHAR {

private
{
    import std.utf;
    import dwt.internal.win32.os;
    import dwt.util.system;
}

	int codePage;
	public wchar[] chars;
	public char[] bytes;
	int byteCount;

//public final static int sizeof = OS.IsUnicode ? 2 : 1;
static int sizeof() { return OS.IsUnicode ? 2 : 1; }

public this(int codePage, int length) {
	this.codePage = codePage;
	if (OS.IsUnicode) {
		chars = new wchar [length];
	} else {
		bytes = new char [byteCount = length];
	}
}



public this(int codePage, wchar ch, bit terminate) {
    wchar[1] bleh;
    bleh[0] = ch;
    this (codePage, bleh, terminate);
}

public this(int codePage, wchar[] string, bit terminate) {
	this.codePage = codePage;
	if (OS.IsUnicode) {
		this.chars = string;
	} else {
		int cp = codePage != 0 ? codePage : OS.CP_ACP;
		bytes = new char [byteCount = charCount * 2 + (terminate ? 1 : 0)];
		byteCount = OS.WideCharToMultiByte (cp, 0, string, string.length, bytes, byteCount, null, null);
		if (terminate) byteCount++;
	}
}


public int length () {
	if (OS.IsUnicode) {
		return chars.length;
	} else {
		return byteCount;
	}
}

public int strlen () {
	if (OS.IsUnicode) {
		for (int i=0; i<chars.length; i++) {
			if (chars [i] == '\0') return i;
		}
		return chars.length;
	} else {
		for (int i=0; i<byteCount; i++) {
			if (bytes [i] == '\0') return i;
		}
		return byteCount;
	}
}

public int tcharAt (int index) {
	if (OS.IsUnicode) {
		return chars [index];
	} else {
		int ch = bytes [index] & 0xFF;
		if (OS.IsDBCSLeadByte (cast(char) ch)) {
			ch = ch << 8 | (bytes [index + 1] & 0xFF);
		}
		return ch;
	}
}

public char[] toString() {
        return toUTF8(toStringw (0, length ()));
}

public wchar[] toString( int start, int length ) {
        return toStringw( start, length );
}

public wchar[] toStringw( int start, int length ) {
	if (OS.IsUnicode) {
		//return new wchar[] (chars, start, length);
                return chars[start..start+length].dup;
	} else {
		char [] bytes = this.bytes;
		if (start != 0) {
			bytes = new char [length];
			System.arraycopy (this.bytes, start, bytes, 0, length);
			bytes[] = this.bytes; // done.  (D is cool)
		}
		wchar [] chars = new wchar [length];
		int cp = codePage != 0 ? codePage : OS.CP_ACP;
		int charCount = OS.MultiByteToWideChar (cp, OS.MB_PRECOMPOSED, bytes, length, chars, length);
		return new wchar[] (chars, 0, charCount);
	}
}


}
