// Written by Christopher E. Miller
// www.dprogramming.com
// Thou may use this code freely at thine own risk
// A lot of code is taken from the origional string.d written by Walter Bright

module dwt.util.string;

import std.string;
import crc32;
import object; //for printf/wprintf


template Strings(T)
{
	alias T schar;

	int toInt(schar[] sc)
	{
		int value = 0;

		foreach (schar c; sc)
		{
			if (c < '0' || c > '9')
				break; // raise NumberFormatException?
			else
				value = (value * 10) + (c - '0');
		}
		return value;
	}

	int toInt(schar[] sc, int radix)
	in { assert(radix >= 2 && radix <= 16); }
	body
	{
		static const schar[] digits = "0123456789ABCDEF";
		int value = 0;

		foreach (schar c; sc)
		{
			int val = ifind(digits, c);
			if (val == -1 || val >= radix)
				break; // raise NumberFormatException?
			else
				value = (value * radix) + val;
		}
		return value;
	}

	private alias toStringz _toStringz;
	schar* toStringz(schar[] sc)
	{
		schar* p = &sc[0] + sc.length;
		if(!*p)
			return sc;

		schar[] result = new schar[sc.length + 1];
		result[0 .. sc.length] = sc;
		return result;
	}


	private alias toString _toString;
	schar[] toString(schar* scz) //takes a null-terminated string
	{
		if(!scz)
			return null;

		uint i;
		for(i = 0;; i++)
		{
			if(!scz[i])
				return scz[0 .. i];
		}
	}


	schar[] toString(uint u) //to a string of memory
	{
		schar[uint.sizeof * 3] buffer;
		int ndigits;
		schar c;
		schar[] result;

		ndigits = 0;
		while(u)
		{
			c = (u % 10) + '0';
			u /= 10;
			ndigits++;
			buffer[buffer.length - ndigits] = c;
		}
		result = new schar[ndigits];
		result[] = buffer[buffer.length - ndigits .. buffer.length];
		return result;
	}


	private alias cmp _cmp;
	int cmp(schar[] s1, schar[] s2)
	{
		uint l = s1.length;
		int result;

		if(s2.length < l)
			l = s2.length;
		result = memcmp(s1, s2, l * schar.sizeof);
		if(result == 0)
			result = cast(int)s1.length - cast(int)s2.length;
		return result;
	}


	private alias icmp _icmp;
	int icmp(schar[] s1, schar[] s2)
	{
		uint len = s1.length;
		int result;

		if(s2.length < len)
			len = s2.length;
		for(int i = 0; i < len; i++)
		{
			if(s1[i] != s2[i])
			{
				schar c1 = s1[i];
				schar c2 = s2[i];

				if(c1 >= cast(schar)'A' && c1 <= cast(schar)'Z')
					c1 += cast(int)'a' - cast(int)'A';
				if(c2 >= 'A' && c2 <= 'Z')
					c2 += cast(int)'a' - cast(int)'A';
				result = cast(int)c1 - cast(int)c2;
				if(result)
					break;
			}
		}
		if(result == 0)
			result = cast(int)s1.length - cast(int)s2.length;
		return result;
	}


	private alias tolower _tolower;
	schar[] tolower(schar[] sc)
	{
		int changed;
		int i;

		changed = 0;
		for(i = 0; i < sc.length; i++)
		{
			schar c = sc[i];
			if(cast(schar)'A' <= c && c <= cast(schar)'Z')
			{
				if(!changed)
				{
					schar[] r = new schar[sc.length];
					r[] = sc;
					sc = r;
					changed = 1;
				}
				sc[i] = c + (cast(schar)'a' - cast(schar)'A');
			}
		}
		return sc;
	}


	schar tolower(schar ch)
	{
		if(cast(schar)'A' <= ch && ch <= cast(schar)'Z')
			return ch + (cast(schar)'a' - cast(schar)'A');
		return ch;
	}


	private alias toupper _toupper;
	schar[] toupper(schar[] sc)
	{
		int changed;
		int i;

		changed = 0;
		for (i = 0; i < sc.length; i++)
		{
			schar c = sc[i];
			if(cast(schar)'a' <= c && c <= cast(schar)'z')
			{
				if (!changed)
				{
					schar[] r = new schar[sc.length];
					r[] = sc;
					sc = r;
					changed = 1;
				}
				sc[i] = c - (cast(schar)'a' - cast(schar)'A');
			}
		}
		return sc;
	}


	schar toupper(schar ch)
	{
		if(cast(schar)'a' <= ch && ch <= cast(schar)'z')
			return ch - (cast(schar)'a' - cast(schar)'A');
		return ch;
	}


	private alias find _find;
	int find(schar[] sc, schar ch)
	{
		int i;
		for(i = 0; i != sc.length; i++)
		{
			if(ch == sc[i])
				return i;
		}
		return -1;
	}


	private alias ifind _ifind;
	int ifind(schar[] sc, schar ch)
	{
		int i;
		for(i = 0; i != sc.length; i++)
		{
			if(toupper(ch) == toupper(sc[i]))
				return i;
		}
		return -1;
	}


	private alias rfind _rfind;
	int rfind(schar[] sc, schar ch)
	{
		int i;
		for (i = sc.length; i-- > 0;)
		{
			if (sc[i] == ch)
			break;
		}
		return i;
	}


	private alias irfind _irfind;
	int irfind(schar[] sc, schar ch)
	{
		int i;
		for (i = sc.length; i-- > 0;)
		{
			if (toupper(sc[i]) == toupper(ch))
			break;
		}
		return i;
	}


	int find(schar[] sc, schar[] sub)
	{
		int sublength = sub.length;

		if(sublength == 0)
			return 0;

		schar c = sub[0];
		if(sublength == 1)
			return find(sc, c);

		int imax = sc.length - sublength + 1;

		// Remainder of sub[]
		schar *q = &sub[1];
		sublength--;

		for(int i = 0; i < imax; i++)
		{
			for(; i < imax; i++)
			{
				if(sc[i] == c)
					goto found_ch;
			}
			break;

			found_ch:
			int j = i + 1;
			if(sub[1 .. 1 + sublength] == sc[j .. j + sublength])
				return i;
		}
		return -1;
	}


	int ifind(schar[] sc, schar[] sub)
	{
		int sublength = sub.length;

		if(sublength == 0)
			return 0;

		schar c = sub[0];
		if(sublength == 1)
			return find(sc, c);

		int imax = sc.length - sublength + 1;

		// Remainder of sub[]
		schar *q = &sub[1];
		sublength--;

		for(int i = 0; i < imax; i++)
		{
			for(; i < imax; i++)
			{
				if(sc[i] == c)
					goto found_ch;
			}
			break;

			found_ch:
			int j = i + 1;
			if(!icmp(sub[1 .. 1 + sublength], sc[j .. j + sublength]))
				return i;
		}
		return -1;
	}


	int smemcmp(schar* sc1, schar* sc2, uint len)
	{
		return cmp(sc1[0 .. len], sc2[0 .. len]);
	}


	int smemicmp(schar* sc1, schar* sc2, uint len)
	{
		return icmp(sc1[0 .. len], sc2[0 .. len]);
	}


	int rfind(schar[] sc, schar[] sub)
	{
		schar c;

		if(sub.length == 0)
			return sc.length;
		c = sub[0];
		for(int i = sc.length - sub.length; i >= 0; i--)
		{
			if(sc[i] == c)
			{
				if(!smemcmp(&sc[i + 1], &sub[1], sub.length - 1))
					return i;
			}
		}
		return -1;
	}


	int irfind(schar[] sc, schar[] sub)
	{
		schar c;

		if(sub.length == 0)
			return sc.length;
		c = sub[0];
		for(int i = sc.length - sub.length; i >= 0; i--)
		{
			if(sc[i] == c)
			{
				if(!smemicmp(&sc[i + 1], &sub[1], sub.length - 1))
					return i;
			}
		}
		return -1;
	}


	private alias startswith _startswith;
	bit startswith(schar[] hay, schar[] findme)
	{
		if(findme.length > hay.length)
			return false;

		uint i;
		for(i = 0;; i++)
		{
			if(hay.length == i)
			{
				if(findme.length == i)
					return true;
				else
					return false;
			}
			if(findme.length == i)
			{
				return true; //matched all of findme
			}

			if(hay[i] != findme[i])
				return false;
		}
	}


	private alias istartswith _istartswith;
	bit istartswith(schar[] hay, schar[] findme)
	{
		if(findme.length > hay.length)
			return false;

		uint i;
		for(i = 0;; i++)
		{
			if(hay.length == i)
			{
				if(findme.length == i)
					return true;
				else
					return false;
			}
			if(findme.length == i)
			{
				return true; //matched all of findme
			}

			if(toupper(hay[i]) != toupper(findme[i]))
				return false;
		}
	}


	private alias endswith _endswith;
	bit endswith(schar[] hay, schar[] findme)
	{
		if(findme.length > hay.length)
			return false;

		uint hi = hay.length - findme.length;
		uint fi = 0;
		while(hi != hay.length)
		{
			if(hay[hi] != findme[fi])
				return false;

			hi++;
			fi++;
		}

		return true; //fell thru and matched all
	}


	private alias iendswith _iendswith;
	bit iendswith(schar[] hay, schar[] findme)
	{
		if(findme.length > hay.length)
			return false;

		uint hi = hay.length - findme.length;
		uint fi = 0;
		while(hi != hay.length)
		{
			if(toupper(hay[hi]) != toupper(findme[fi]))
				return false;

			hi++;
			fi++;
		}

		return true; //fell thru and matched all
	}


	private alias findany _findany;
	int findany(schar[] hay, schar[] findme) //find any schar of findme in hay
	{
		uint i;
		for(i = 0; i != hay.length; i++)
		{
			foreach(schar ch; findme)
			{
				if(ch == hay[i])
					return i;
			}
		}
		return -1;
	}


	private alias ifindany _ifindany;
	int ifindany(schar[] hay, schar[] findme) //find any schar of findme in hay
	{
		uint i;
		for(i = 0; i != hay.length; i++)
		{
			foreach(schar ch; findme)
			{
				if(toupper(ch) == toupper(hay[i]))
					return i;
			}
		}
		return -1;
	}


	int findany(schar[] hay, schar[][] findme) //find any schar[] (substring) of findme in hay
	{
		uint i;
		for(i = 0; i != hay.length; i++)
		{
			foreach(schar[] s; findme)
			{
				if(startswith(hay[i .. hay.length], s))
					return i;
			}
		}
		return -1;
	}


	int ifindany(schar[] hay, schar[][] findme) //find any schar[] (substring) of findme in hay
	{
		uint i;
		for(i = 0; i != hay.length; i++)
		{
			foreach(schar[] s; findme)
			{
				if(istartswith(hay[i .. hay.length], s))
					return i;
			}
		}
		return -1;
	}


	private alias count _count;
	int count(schar[] sc, schar[] sub)
	{
		int i;
		int j;
		int count = 0;

		for (i = 0; i < sc.length; i += j + sub.length)
		{
			j = find(sc[i .. sc.length], sub);
			if(j == -1)
				break;
			count++;
		}
		return count;
	}


	private alias replace _replace;
	schar[] replace(schar[] sc, schar[] from, schar[] to)
	{
		schar[] p;
		int i;
		int istart;

		istart = 0;
		while(istart < sc.length)
		{
			i = find(sc[istart .. sc.length], from);
			if(i == -1)
			{
				p ~= sc[istart .. sc.length];
				break;
			}
			p ~= sc[istart .. istart + i];
			p ~= to;
			istart += i + from.length;
		}
		return p;
	}


	private alias replaceSlice _replaceSlice;
	schar[] replaceSlice(schar[] string, schar[] slice, schar[] replacement)
	in
	{
		// Verify that slice[] really is a slice of string[]
		//int so = (schar*)slice - (schar*)string;
		int so = cast(schar*)slice - cast(schar*)string;
		
		assert(so >= 0);
		assert(string.length >= so + slice.length);
	}
	body
	{
		schar[] result;
		int so = cast(schar*)slice - cast(schar*)string;

		result.length = string.length - slice.length + replacement.length;

		result[0 .. so] = string[0 .. so];
		result[so .. so + replacement.length] = replacement;
		result[so + replacement.length .. result.length] = string[so + slice.length .. string.length];

		return result;
	}


	private alias insert _insert;
	schar[] insert(schar[] sc, int index, schar[] sub)
	in
	{
		assert(0 <= index && index <= sc.length);
	}
	body
	{
		if(sub.length == 0)
			return sc;

		if(sc.length == 0)
			return sub;

		int newlength = sc.length + sub.length;
		schar[] result = new schar[newlength];

		result[0 .. index] = sc[0 .. index];
		result[index .. index + sub.length] = sub;
		result[index + sub.length .. newlength] = sc[index .. sc.length];
		return result;
	}


	private alias split _split;
	schar[][] split(schar[] sc)
	{
		uint i;
		uint istart;
		int inword;
		schar[][] words;

		inword = 0;
		for (i = 0; i < sc.length; i++)
		{
			switch(sc[i])
			{
				case ' ':
				case '\t':
				case '\f':
				case '\r':
				case '\n':
				case '\v':
					if(inword)
					{
						words ~= sc[istart .. i];
						inword = 0;
					}
					break;

				default:
					if(!inword)
					{
						istart = i;
						inword = 1;
					}
					break;
			}
		}
		if(inword)
			words ~= sc[istart .. i];
		return words;
	}


	private alias smemchr _smemchr;
	schar* smemchr(schar* smem, schar chr, uint len)
	{
		schar* end = smem + len;
		while(smem != end)
		{
			if(*smem == chr)
				return smem;
			smem++;
		}
		return null;
	}


	schar[][] split(schar[] sc, schar[] delim)
	in
	{
		assert(delim.length > 0);
	}
	body
	{
		uint i;
		uint j;
		schar[][] words;

		i = 0;
		if(sc.length)
		{
			if(delim.length == 1)
			{
				char c = delim[0];
				uint nwords = 0;
				schar *p = &sc[0];
				schar *pend = p + sc.length;

				while(true)
				{
					nwords++;
					p = smemchr(p, c, pend - p);
					if(!p)
						break;
					p++;
					if(p == pend)
					{
						nwords++;
						break;
					}
				}
				words.length = nwords;

				int wordi = 0;
				i = 0;
				while(true)
				{
					p = smemchr(&sc[i], c, sc.length - i);
					if(!p)
					{
						words[wordi] = sc[i .. sc.length];
						break;
					}
					j = p - &sc[0];
					words[wordi] = sc[i .. j];
					wordi++;
					i = j + 1;
					if(i == sc.length)
					{
						words[wordi] = "";
						break;
					}
				}
				assert(wordi + 1 == nwords);
			}
			else
			{
				uint nwords = 0;

				while(true)
				{
					nwords++;
					j = find(sc[i .. sc.length], delim);
					if(j == -1)
						break;
					i += j + delim.length;
					if(i == sc.length)
					{
						nwords++;
						break;
					}
					assert(i < sc.length);
				}
				words.length = nwords;

				int wordi = 0;
				i = 0;
				while(true)
				{
					j = find(sc[i .. sc.length], delim);
					if(j == -1)
					{
						words[wordi] = sc[i .. sc.length];
						break;
					}
					words[wordi] = sc[i .. i + j];
					wordi++;
					i += j + delim.length;
					if(i == sc.length)
					{
						words[wordi] = "";
						break;
					}
					assert(i < sc.length);
				}
				assert(wordi + 1 == nwords);
			}
		}
		return words;
	}


	private alias splitlines _splitlines;
	schar[][] splitlines(schar[] sc)
	{
		uint i;
		uint istart;
		uint nlines;
		schar[][] lines;

		nlines = 0;
		for (i = 0; i < sc.length; i++)
		{
			schar c;

			c = sc[i];
			if(c == '\r' || c == '\n')
			{
				nlines++;
				istart = i + 1;
				if(c == '\r' && i + 1 < sc.length && sc[i + 1] == '\n')
				{
					i++;
					istart++;
				}
			}
		}
		if(istart != i)
			nlines++;

		lines = new schar[][nlines];
		nlines = 0;
		istart = 0;
		for(i = 0; i < sc.length; i++)
		{
			schar c;

			c = sc[i];
			if(c == '\r' || c == '\n')
			{
				lines[nlines] = sc[istart .. i];
				nlines++;
				istart = i + 1;
				if(c == '\r' && i + 1 < sc.length && sc[i + 1] == '\n')
				{
					i++;
					istart++;
				}
			}
		}
		if(istart != i)
		{
			lines[nlines] = sc[istart .. i];
			nlines++;
		}

		assert(nlines == lines.length);
		return lines;
	}


	private alias join _join;
	schar[] join(schar[][] words, schar[] sep)
	{
		uint len;
		uint seplen;
		uint i;
		uint j;
		schar[] result;

		if(words.length)
		{
			len = 0;
			for(i = 0; i < words.length; i++)
				len += words[i].length;

			seplen = sep.length;
			len += (words.length - 1) * seplen;

			result = new schar[len];

			i = 0;
			while(true)
			{
				uint wlen = words[i].length;

				result[j .. j + wlen] = words[i];
				j += wlen;
				i++;
				if(i >= words.length)
					break;
				result[j .. j + seplen] = sep;
				j += seplen;
			}
			assert(j == len);
		}
		return result;
	}


	private alias stripl _stripl;
	schar[] stripl(schar[] sc)
	{
		uint i;
		for(i = 0; i < sc.length; i++)
		{
			if(!iswhite(sc[i]))
				break;
		}
		return sc[i .. sc.length];
	}

	private alias stripr _stripr;
	schar[] stripr(schar[] sc)
	{
		uint i;
		for(i = sc.length; i > 0; i--)
		{
			if(!iswhite(sc[i - 1]))
				break;
		}
		return sc[0 .. i];
	}

	private alias strip _strip;
	schar[] strip(schar[] sc)
	{
		return stripr(stripl(sc));
	}


	private alias expandtabs _expandtabs;
	schar[] expandtabs(schar[] sc, int tabsize)
	{
		schar[] r;
		int i;
		int istart;
		int col;
		//static schar[8] spaces = "        "; //error: cannot implicitly convert char[8] to wchar
		static schar[] spaces = "        ";

		col = 0;
		for (i = 0; i < sc.length; i++)
		{
			schar c;

			c = sc[i];
			if(c == '\t')
			{
				int tabstop;

				r ~= sc[istart .. i];
				istart = i + 1;

				tabstop = col + tabsize;
				tabstop -= tabstop % tabsize;
				while(col < tabstop)
				{
					int n = tabstop - col;
					if (n > spaces.length)
					n = spaces.length;
					r ~= spaces[0 .. n];
					col += n;
				}
			}
			else
			{
				col++;
			}
		}
		r ~= sc[istart .. i];
		return r;
	}


	private alias ljustify _ljustify;
	schar[] ljustify(schar[] sc, int width)
	{
		if(sc.length >= width)
			return sc;
		schar[] r = new schar[width];
		r[0 .. sc.length] = sc;
		r[sc.length .. width] = cast(schar)' ';
		return r;
	}


	private alias rjustify _rjustify;
	schar[] rjustify(schar[] sc, int width)
	{
		if(sc.length >= width)
			return sc;
		schar[] r = new schar[width];
		r[0 .. width - sc.length] = cast(schar)' ';
		r[width - sc.length .. width] = sc;
		return r;
	}

	private alias center _center;
	schar[] center(schar[] sc, int width)
	{
		if(sc.length >= width)
			return sc;
		schar[] r = new schar[width];
		int left = (width - sc.length) / 2;
		r[0 .. left] = cast(schar)' ';
		r[left .. left + sc.length] = sc;
		r[left + sc.length .. width] = cast(schar)' ';
		return r;
	}


	private alias zfill _zfill;
	schar[] zfill(schar[] sc, int width)
	{
		if(sc.length >= width)
			return sc;
		schar[] r = new schar[width];
		r[0 .. width - sc.length] = cast(schar)'0';
		r[width - sc.length .. width] = sc;
		return r;
	}


	private alias capitalize _capitalize;
	schar[] capitalize(schar[] sc)
	{
		if(sc.length)
		{
			schar c = sc[0];
			if('a' <= c && c <= 'z')
			{
				schar[] r = new schar[sc.length];
				r[] = sc;
				sc = r;
				sc[0] = c - (cast(schar)'a' - 'A');
			}
		}
		return sc;
	}


	private alias capwords _capwords;
	schar[] capwords(schar[] sc)
	{
		schar[] r;
		int inword;
		int i;
		int istart;

		istart = 0;
		inword = 0;
		for (i = 0; i < sc.length; i++)
		{
			switch(sc[i])
			{
				case ' ':
				case '\t':
				case '\f':
				case '\r':
				case '\n':
				case '\v':
					if(inword)
					{
						r ~= sc[istart .. i];
						inword = 0;
					}
					break;

				default:
					if(!inword)
					{
						if(r.length)
							r ~= ' ';
						istart = i;
						inword = 1;
					}
					break;
			}
		}
		if(inword)
		{
			r ~= sc[istart .. i];
		}

		// Go back through r and capitalize the words
		inword = 0;
		for (i = 0; i < r.length; i++)
		{
			schar c = r[i];

			if(c >= 'A' && c <= 'Z')
			{
				inword = 1;
			}
			else if(c >= 'a' && c <= 'z')
			{
				if(inword == 0)
				{
					c -= cast(schar)'a' - 'A';
					r[i] = c;
				}
				inword = 1;
			}
			else
			inword = 0;
		}

		return r;
	}


	private alias toHash _toHash;
	uint toHash(schar[] sc)
	{
		return strcrc32(cast(char[])sc);
	}


	class String
	{
		schar[] data = null;
		uint len = 0;


		this()
		{
		}


		this(schar[] data)
		{
			this.data = data;
			len = data.length;
		}


		this(String str)
		{
			data = str.data;
			len = data.length;
		}


		String dup() //getter
		{
			return new String(opSlice().dup);
		}


		uint length() //getter
		{
			return len;
		}


		void length(uint newLen) //setter
		{
			if(newLen > data.length)
			{
				data.length = newLen;
				len = newLen;
			}
			else
			{
				len = newLen;
			}
		}


		uint reserve() //getter
		{
			return data.length;
		}


		void reserve(uint length) //setter
		{
			if(length >= len)
				data.length = length;
		}


		//sets the data
		//make sure the new data isn't part of the old
		//if it is, just construct a new object
		//this is to continue using the same memory and same reserve (if possible)
		String opCall(schar[] sc)
		{
			if(sc.length <= data.length)
			{
				data[0 .. sc.length] = sc;
				len = sc.length;
			}
			else
			{
				data = sc.dup;
				len = sc.length;
			}
			return this;
		}


		String opCall(String str)
		{
			return opCall(str[]);
		}


		override uint toHash()
		{
			return _toHash(opSlice());
		}


		override void print()
		{
			if(schar.sizeof == char.sizeof)
				printf("%.*s\n", opSlice());
			else if(schar.sizeof == wchar.sizeof)
				wprintf("%.*s\n", opSlice());
			else
				printf("String %p\n", this);
		}


		override char[] toString() //to a string of memory, not String class :)
		{
			if(schar.sizeof == char.sizeof)
				return cast(char[])opSlice();

			char[] result = new char[len];
			uint i;
			for(i = 0; i != len; i++)
			{
				if(data[i] > char.max)
					result[i] = '?'; //might not be this character
				else
					result[i] = cast(char)data[i];
			}

			return result;
		}


		schar* toStringz() //more efficient used with this String class than _toStringz
		{
			if(len == data.length) //need to make room for null in reserve
			{
				data.length = data.length + 1;
				data[len] = 0;
			}
			else
			{
				//if(data[len]) //a harmless assignment is more efficient than testing, right?
					data[len] = 0;
			}

			return &data[0];
		}


		schar[] opSlice()
		{
			return data[0 .. len];
		}


		schar[] opSlice(uint x, uint y)
		{
			return opSlice()[x .. y]; //so indexing the reserve is out of bounds
		}


		schar opIndex(uint i)
		{
			return opSlice()[i]; //so indexing the reserve is out of bounds
		}


		String opCatAssign(String str)
		{
			return opCatAssign(str[]);
		}


		String opCatAssign(schar[] sc)
		{
			if(data.length < sc.length + len)
				data.length = sc.length + len;
			data[len .. len + sc.length] = sc;
			len += sc.length;
			return this;
		}


		String opCatAssign(schar ch)
		{
			return opCatAssign((&ch)[0 .. 1]);
		}


		String opCatAssign(uint u)
		{
			return opCatAssign(_toString(u));
		}


		String opCat(String str)
		{
			return new String(opSlice() ~ str[]);
		}


		String opCat(schar[] sc)
		{
			return new String(opSlice() ~ sc);
		}


		String opCat(uint u)
		{
			return opCat(_toString(u));
		}


		String opCat(schar ch)
		{
			return new String(opSlice() ~ (&ch)[0 .. 1]);
		}


		String opCat_r(String str)
		{
			return new String(str[] ~ opSlice());
		}


		String opCat_r(schar[] sc)
		{
			return new String(sc ~ opSlice());
		}


		String opCat_r(schar ch)
		{
			return new String((&ch)[0 .. 1] ~ opSlice());
		}


		String opCat_r(uint u)
		{
			return opCat_r(_toString(u));
		}


		override int opEquals(Object str)
		{
			assert(cast(String)str);
			return opEquals(cast(String)str);
		}


		int opEquals(String str)
		{
			return opSlice() == str[];
		}


		int opEquals(schar[] sc)
		{
			return opSlice() == sc;
		}


		alias opCmp cmp;
		override int opCmp(Object str)
		{
			assert(cast(String)str);
			return opCmp(cast(String)str);
		}


		int opCmp(String str)
		{
			return opCmp(str[]);
		}


		int opCmp(schar[] sc)
		{
			return _cmp(opSlice(), sc);
		}


		int icmp(String str)
		{
			return icmp(str[]);
		}


		int icmp(schar[] sc)
		{
			return _icmp(opSlice(), sc);
		}


		String tolower() //caller does not own returned string
		{
			schar[] sc = _tolower(opSlice());
			return (&sc[0] == &data[0]) ? this : new String(sc);
		}


		String toupper() //caller does not own returned string
		{
			schar[] sc = _toupper(opSlice());
			return (&sc[0] == &data[0]) ? this : new String(sc);
		}


		int find(schar ch)
		{
			return _find(opSlice(), ch);
		}


		int ifind(schar ch)
		{
			return _ifind(opSlice(), ch);
		}


		int rfind(schar ch)
		{
			return _rfind(opSlice(), ch);
		}


		int irfind(schar ch)
		{
			return _irfind(opSlice(), ch);
		}


		int find(schar[] sub)
		{
			return _find(opSlice(), sub);
		}


		int ifind(schar[] sub)
		{
			return _ifind(opSlice(), sub);
		}


		int find(String sub)
		{
			return find(sub[]);
		}


		int ifind(String sub)
		{
			return ifind(sub[]);
		}


		int rfind(schar[] sub)
		{
			return _rfind(opSlice(), sub);
		}


		int irfind(schar[] sub)
		{
			return _irfind(opSlice(), sub);
		}


		int rfind(String sub)
		{
			return rfind(sub[]);
		}


		int irfind(String sub)
		{
			return irfind(sub[]);
		}


		bit startswith(schar[] findme)
		{
			return _startswith(opSlice(), findme);
		}


		bit startswith(String findme)
		{
			return _startswith(opSlice(), findme[]);
		}


		bit istartswith(schar[] findme)
		{
			return _istartswith(opSlice(), findme);
		}


		bit istartswith(String findme)
		{
			return _istartswith(opSlice(), findme[]);
		}


		bit endswith(schar[] findme)
		{
			return _endswith(opSlice(), findme);
		}


		bit endswith(String findme)
		{
			return _endswith(opSlice(), findme[]);
		}


		bit iendswith(schar[] findme)
		{
			return _iendswith(opSlice(), findme);
		}


		bit iendswith(String findme)
		{
			return _iendswith(opSlice(), findme[]);
		}


		int findany(schar[] findme) //find any schar of findme in hay
		{
			return _findany(opSlice(), findme);
		}


		int findany(String findme) //find any schar of String findme in hay
		{
			return _findany(opSlice(), findme[]);
		}


		int ifindany(schar[] findme) //find any schar of findme in hay
		{
			return _ifindany(opSlice(), findme);
		}


		int ifindany(String findme) //find any schar of String findme in hay
		{
			return _ifindany(opSlice(), findme[]);
		}


		int count(schar[] sub)
		{
			return _count(opSlice(), sub);
		}


		int count(String sub)
		{
			return count(sub[]);
		}


		String replace(schar[] from, schar[] to)
		{
			return new String(_replace(opSlice(), from, to));
		}


		String replace(String from, String to)
		{
			return replace(from[], to[]);
		}


		String replaceSlice(schar[] slice, schar[] replacement)
		{
			return new String(_replaceSlice(opSlice(), slice, replacement));
		}


		String replaceSlice(schar[] slice, String replacement)
		{
			return replaceSlice(slice, replacement[]);
		}


		String insert(int index, schar[] sub)
		{
			return new String(_insert(opSlice(), index, sub));
		}


		String insert(int index, String sub)
		{
			return new String(_insert(opSlice(), index, sub[]));
		}


		static String[] convert(schar[][] scs)
		{
			String[] result;
			if(!scs)
				return null;
			result = new String[scs.length];
			uint i;
			for(i = 0; i != result.length; i++)
			{
				result[i] = new String(scs[i]);
			}
			return result;
		}


		String[] split() //splits up words
		{
			return convert(_split(opSlice()));
		}


		String[] split(schar[] delim)
		{
			return convert(_split(opSlice(), delim));
		}


		String[] split(String delim)
		{
			return split(delim[]);
		}


		String[] splitlines()
		{
			return convert(_splitlines(opSlice()));
		}


		String stripl()
		{
			schar[] sc = _stripl(opSlice());
			return (sc.length == len) ? this : new String(sc);
		}


		String stripr()
		{
			schar[] sc = _stripr(opSlice());
			return (sc.length == len) ? this : new String(sc);
		}


		String strip()
		{
			schar[] sc = _strip(opSlice());
			return (sc.length == len) ? this : new String(sc);
		}


		String expandtabs(int tabsize)
		{
			return new String(_expandtabs(opSlice(), tabsize));
		}


		String ljustify(int width)
		{
			schar[] sc = _ljustify(opSlice(), width);
			return (sc.length == len) ? this : new String(sc);
		}


		String rjustify(int width)
		{
			schar[] sc = _rjustify(opSlice(), width);
			return (sc.length == len) ? this : new String(sc);
		}


		String center(int width)
		{
			schar[] sc = _center(opSlice(), width);
			return (sc.length == len) ? this : new String(sc);
		}


		String zfill(int width)
		{
			schar[] sc = _zfill(opSlice(), width);
			return (sc.length == len) ? this : new String(sc);
		}


		String capitalize()
		{
			schar[] sc = _capitalize(opSlice());
			return (&sc[0] == &data[0]) ? this : new String(sc);
		}


		String capwords()
		{
			return new String(_capwords(opSlice()));
		}


		static String join(schar[][] words, schar[] sep)
		{
			return new String(_join(words, sep));
		}


		static String join(String[] words, String sep)
		{
			//this isn't very good but it is better than duplicating join's code?
			schar[][] scs = new schar[][words.length];
			uint i;
			for(i = 0; i != words.length; i++)
			{
				scs[i] = words[i][];
			}
			return new String(_join(scs, sep[]));
		}
	}
}


alias Strings!(char) StringInstance;
alias Strings!(wchar) WStringInstance;
alias Strings!(dchar) DStringInstance;

alias StringInstance.String String;
alias WStringInstance.String WString;
alias DStringInstance.String DString;


unittest
{
	String str = new String("hello");
	str ~= " world";
	assert(str == "hello world");

	assert((new String("a")) < "b");
	assert((new String("b")) <= "b");
	assert((new String("b")) >= "b");
	assert((new String("c")) > "b");

	assert(('2' ~ str) == "2hello world");
	assert((cast(uint)2 ~ str) == "2hello world");

	str = new String("Hello");
	assert(str.tolower() == "hello");
	assert(str.toupper() == "HELLO");
	str("hello");
	assert(str.tolower() === str);

	WString wstr = new WString("this is a WString test");

	assert(wstr.find('h') == 1);
	assert(wstr.find('\0') == -1);
	assert(wstr.find("") == 0);
	assert(wstr.find("i") == 2);
	assert(wstr.find("is") == 2);
	assert(wstr.find("testbad") == -1);
	assert(wstr.find("test\0") == -1);
	assert(wstr.find("test") != -1);
	assert(wstr.find("wstring") == -1);
	assert(wstr.find("WString") != -1);

	assert(wstr.rfind('t') != -1);
	assert(wstr.rfind('W') != -1);
	assert(wstr.rfind('Z') == -1);
	assert(wstr.rfind("is is a") != -1);
	assert(wstr.rfind("is is a") == 2);
	assert(wstr.rfind("testbad") == -1);
	assert(wstr.rfind("test\0") == -1);
	assert(wstr.rfind("test") != -1);

	assert(wstr.count("t") == 4);

	str = new String;
	str.reserve(32);
	str("This is a foo foo list");
	assert(str.reserve() == 32);
	str = str.replace("foo", "silly");
	assert(str == "This is a silly silly list");

	str("hello");
	char[] slice = str[2 .. 4];
	str = str.replaceSlice(slice, "bar");
	assert(str == "hebaro");

	assert(wstr("abcd").insert(0, "e") == "eabcd");

	wstr("toString");
	assert(wstr.toString() == cast(char[])"toString");

	str(" peter paul\tjerry ");
	String[] strs;
	strs = str.split();
	assert(strs.length == 3);
	assert(strs[0] == "peter");
	assert(strs[1] == "paul");
	assert(strs[2] == "jerry");

	DString dstr = new DString(",peter,paul,jerry,");
	DString[] dstrs = dstr.split(",");
	assert(dstrs.length == 5);
	assert(dstrs[0] == "");
	assert(dstrs[1] == "peter");
	assert(dstrs[2] == "paul");
	assert(dstrs[3] == "jerry");
	assert(dstrs[4] == "");

	str("\rpeter\n\rpaul\r\njerry\n");
	strs = str.splitlines();
	assert(strs.length == 5);
	assert(strs[0].length == 0);
	assert(strs[1] == "peter");
	assert(strs[2].length == 0);
	assert(strs[3] == "paul");
	assert(strs[4] == "jerry");

	str("hi");
	assert(str.strip() === str);
	str(" \t\r hi\n");
	assert(str.strip() == "hi");

	dstr("This \tis\t a fofof\tof list");
	assert(dstr.expandtabs(8) == "This    is       a fofof        of list");

	wstr("hello");
	assert(wstr.ljustify(8) == "hello   ");
	assert(wstr.rjustify(8) == "   hello");
	assert(wstr.center(8) == " hello  ");
	assert(wstr.zfill(8) == "000hello");
	assert(wstr.center(0) === wstr);

	dstr("foL");
	assert(dstr.capitalize() == "FoL");

	str("\tfoo abc(aD)*  \t  (q P  ");
	assert(str.capwords() == "Foo Abc(AD)* (Q P");

	static wchar[][3] words = ["peter", "paul", "jerry"];
	assert(WString.join(words, ",") == "peter,paul,jerry");
	DString[3] dstrWords;
	dstrWords[0] = new DString("peter");
	dstrWords[1] = new DString("paul");
	dstrWords[2] = new DString("jerry");
	assert(DString.join(dstrWords, new DString(":")) == "peter:paul:jerry");

	assert(dstr("abc").cmp("abc") == 0);
	assert(dstr(cast(dchar[])null).cmp(cast(dchar[])null) == 0);
	assert(dstr("").cmp("") == 0);
	assert(dstr("abc").cmp("abcd") < 0);
	assert(dstr("abcd").cmp("abc") > 0);
	assert(dstr("abc").cmp("abd") < 0);
	assert(dstr("bbc").cmp("abc") > 0);

	assert(dstr("aBc").icmp("abc") == 0);
	assert(dstr(cast(dchar[])null).icmp(cast(dchar[])null) == 0);
	assert(dstr("").icmp("") == 0);
	assert(dstr("aBc").icmp("abcd") < 0);
	assert(dstr("aBcd").icmp("abc") > 0);
	assert(dstr("aBc").icmp("abd") < 0);
	assert(dstr("bbc").icmp("abc") > 0);

	assert(wstr("hello").toHash() != init_crc32());

	wstr("hey");
	assert(WStringInstance.toString(wstr.toStringz()) == "hey");
}


alias StringInstance.startswith startswith;
alias StringInstance.istartswith istartswith;
alias StringInstance.endswith endswith;
alias StringInstance.iendswith iendswith;
alias StringInstance.findany findany;
alias StringInstance.ifindany ifindany;
alias StringInstance.irfind irfind;
alias StringInstance.ifind ifind;


unittest
{
	assert(endswith("foo", "o"));
	assert(endswith("hello world", "world"));
	assert(iendswith("Hello World", "World"));
	assert(!endswith("foo", "b"));
	assert(!endswith("foo", "foooo"));
	assert(iendswith("Foo", "foo"));

	assert(findany("foo", "dou") != -1);
	assert(findany("Foo", "fOO") == -1);
	assert(ifindany("Foo", "fbar") != -1);
	assert(ifindany("asdf", "jkl;") == -1);

	const char[][] sa = ["Foo", "bar", "baz"];
	assert(findany("foo", sa) == -1);
	assert(ifindany("Bar", sa) != -1);
	assert(findany("hey bar!", sa) != -1);
}


/*
	these could be placed in string.d to eliminate duplicated code:
alias StringInstance.cmp cmp;
alias StringInstance.icmp icmp;
alias StringInstance.find find;
alias StringInstance.rfind rfind;
alias StringInstance.tolower tolower;
alias StringInstance.toupper toupper;
alias StringInstance.capitalize capitalize;
alias StringInstance.capwords capwords;
alias StringInstance.join join;
alias StringInstance.split split;
alias StringInstance.splitlines splitlines;
alias StringInstance.stripl stripl;
alias StringInstance.stripr stripr;
alias StringInstance.center center;
alias StringInstance.zfill zfill;
alias StringInstance.replace replace;
alias StringInstance.replaceSlice replaceSlice;
alias StringInstance.insert insert;
alias StringInstance.count count;
alias StringInstance.expandtabs expandtabs;
alias StringInstance.toString toString;
alias StringInstance.toStringz toStringz;
*/