/*******************************************************************************
 * Copyright (c) 2000, 2003 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

/*******************************************************************************
 * DWT: SWT port for the D Language
 * 
 * Contributors:
 * 	The DWT team at http://www.dsource.org/projects/dwt
 * 
 *******************************************************************************/

// based on the Event.java class

module dwt.widgets.event;

class Event {

	private import dwt.graphics.gc;
	private import dwt.graphics.rectangle;
	
	private import dwt.util.util;
//	private import dwt.util.string;
	
	private import dwt.widgets.display;
	private import dwt.widgets.widget;

	// The display where the event occurred
	public Display display;
	// The widget that issued the event
	public Widget  widget;
	// The type of event as defined by the event type constants
	public int type;
	// The event specified detail field, as defined by the detail constants
	public int detail;
	// The item that the event occured in (can be null)
	public Widget  item;
	// The graphics context to use when painting that is configured to use the 
	// colors, font and damaged region of the control. It is valid only 
	// during the paint and must not be disposed
	public GC gc;
	// depending on the event type, the x offset of the bounding rectangle
	// of the region that requires painting or the widget-relative, 
	// coordinate of the pointer at the time the mouse button was pressed or
	// released
	public int x;
	public int y;
	// The width of the bounding rectangle of the region that requires painting
	public int width;
	// The height of the bounding rectangel of the region that requires painting
	public int height;
	// The number of following paint events which are pending which may always be
	// zero on some platforms
	public int count;
	// The time that the event occurred
//	public uint _count;
	public uint time;
	// The button that was pressed or released; 1 for the first button, 2 for the
	// second button, and 3 for the third button, etc/
	public int button;
	// Depending on the event, the character represented by the key that was typed.
	public char character;
	// Depending on the event, the key code of the key that was typed, as defined by
	// the key code constants in class SWT
	public int keyCode;
	/**
	 * depending on the event, the state of the keyboard modifier
	 * keys and mouse masks at the time the event was generated.
	 */
	public int stateMask;
	
	// Depending on the event, the range of text being modified.
	public int start, end;
	// Depending on the event, the new text that will be inserted.  Setting this
	// field will change the text that is about to be inserted or deleted.
	public String text;
	// Depending on the event, a flag indicating whether the operation should be
	// allowed.  Setting this field to false will cancel the operation.
	public bit doit=true;
	// A field for application use
	public Object data;
	
	public Rectangle getBounds() {
		return new Rectangle (x, y, width, height);
	}

	public void setBounds(Rectangle rect) {
		x = rect.x;
		y = rect.y;
		width = rect.width;
		height = rect.height;
	}
	
	public char[] toString () { 
		/* TODO: need to implement */ 
		return "some event";
	}


}
	
	

